@@ -459,14 +459,14 @@ get_tmp_outfile (wchar_t *name, HANDLE *outHandle)
 
   auto outName = tmpPath + utf8Name;
 
-  log_mime_parser("%s:%s: Attachment candidate is %s",
+  log_data("%s:%s: Attachment candidate is %s",
                   SRCNAME, __func__, outName.c_str ());
 
   int tries = 1;
   while ((*outHandle = CreateFileUtf8 (outName.c_str ())) == INVALID_HANDLE_VALUE)
     {
       log_debug_w32 (-1, "%s:%s: Failed to open candidate '%s'",
-                     SRCNAME, __func__, outName.c_str());
+                     SRCNAME, __func__, anonstr (outName.c_str()));
 
       char *outNameC = xstrdup (outName.c_str());
 
@@ -885,7 +885,14 @@ store_extension_value (const char *key, const char *val)
 int
 load_extension_value (const char *key, char **val)
 {
-  return load_config_value (HKEY_CURRENT_USER, GPGOL_REGPATH, key, val);
+  int ret = load_config_value (HKEY_CURRENT_USER, GPGOL_REGPATH, key, val);
+  if (val)
+    {
+      log_debug ("%s:%s: LoadReg '%s' val '%s'",
+                 SRCNAME, __func__, key ? key : "null",
+                 *val ? *val : "null");
+    }
+  return ret;
 }
 
 int
