@@ -30,6 +30,8 @@
 
 #include "xmalloc.h"
 
+#include "debug.h"
+
 #include "memdbg.h"
 
 #ifdef HAVE_W32_SYSTEM
@@ -198,8 +200,12 @@ struct
   int automation;            /* General automation */
   int autotrust;             /* TOFU configured for GpgOL */
   int sync_enc;              /* Disabed async encryption */
+  int sync_dec;              /* Disabed async decryption */
   int prefer_smime;          /* S/MIME prefered when autoresolving */
   int smime_html_warn_shown; /* Flag to save if unsigned smime warning was shown */
+  int autoretrieve;           /* Use --auto-key-retrieve. */
+  int search_smime_servers;  /* Search for S/MIME keys on all configured S/MIME keyservers
+                                for each new unknown mail */
 
   /* The forms revision number of the binary.  */
   int forms_revision;
@@ -217,22 +223,6 @@ struct b64_state_s
 };
 typedef struct b64_state_s b64_state_t;
 
-/* Bit values used for extra log file verbosity.  Value 1 is reserved
-   to enable debug menu options.  */
-#define DBG_IOWORKER       (1<<1)
-#define DBG_IOWORKER_EXTRA (1<<2)
-#define DBG_FILTER         (1<<3)
-#define DBG_FILTER_EXTRA   (1<<4)
-#define DBG_MEMORY         (1<<5)
-#define DBG_COMMANDS       (1<<6)
-#define DBG_MIME_PARSER    (1<<7)
-#define DBG_MIME_DATA      (1<<8)
-#define DBG_OOM            (1<<9)
-#define DBG_OOM_EXTRA      (1<<10)
-
-/* Macros to used in conditionals to enable debug output.  */
-#define debug_commands    (opt.enable_debug & DBG_COMMANDS)
-
 size_t qp_decode (char *buffer, size_t length, int *r_slbrk);
 char *qp_encode (const char *input, size_t length, size_t* outlen);
 void b64_init (b64_state_t *state);
@@ -259,46 +249,6 @@ char *trim_trailing_spaces (char *string);
               while(*_vptr) { *_vptr=0; _vptr++; } \
                   } while(0)
 
-#define debug_oom        ((opt.enable_debug & DBG_OOM) || \
-                          (opt.enable_debug & DBG_OOM_EXTRA))
-#define debug_oom_extra  (opt.enable_debug & DBG_OOM_EXTRA)
-void log_debug (const char *fmt, ...) __attribute__ ((format (printf,1,2)));
-void log_error (const char *fmt, ...) __attribute__ ((format (printf,1,2)));
-void log_vdebug (const char *fmt, va_list a);
-void log_debug_w32 (int w32err, const char *fmt,
-                    ...) __attribute__ ((format (printf,2,3)));
-void log_error_w32 (int w32err, const char *fmt,
-                    ...) __attribute__ ((format (printf,2,3)));
-void log_hexdump (const void *buf, size_t buflen, const char *fmt,
-                  ...)  __attribute__ ((format (printf,3,4)));
-
-#define log_oom if (opt.enable_debug & DBG_OOM) log_debug
-#define log_oom_extra if (opt.enable_debug & DBG_OOM_EXTRA) log_debug
-#define log_mime_parser if (opt.enable_debug & DBG_MIME_PARSER) log_debug
-#define log_mime_data if (opt.enable_debug & DBG_MIME_DATA) log_debug
-
-#define gpgol_release(X) \
-{ \
-  if (X && opt.enable_debug & DBG_OOM_EXTRA) \
-    { \
-      log_debug ("%s:%s:%i: Object: %p released ref: %lu \n", \
-                 SRCNAME, __func__, __LINE__, X, X->Release()); \
-      memdbg_released (X); \
-    } \
-  else if (X) \
-    { \
-      X->Release(); \
-    } \
-}
-
-const char *log_srcname (const char *s);
-#define SRCNAME log_srcname (__FILE__)
-
-#define TRACEPOINT log_debug ("%s:%s:%d: tracepoint\n", \
-                              SRCNAME, __func__, __LINE__);
-
-const char *get_log_file (void);
-void set_log_file (const char *name);
 void set_default_key (const char *name);
 
 /*-- Convenience macros. -- */
@@ -364,16 +314,6 @@ _gpgol_stpcpy (char *a, const char *b)
 #define stpcpy(a,b) _gpgol_stpcpy ((a), (b))
 #endif /*!HAVE_STPCPY*/
 
-#ifdef _WIN64
-#define SIZE_T_FORMAT "%I64u"
-#else
-# ifdef HAVE_W32_SYSTEM
-#  define SIZE_T_FORMAT "%u"
-# else
-#  define SIZE_T_FORMAT "%lu"
-# endif
-#endif
-
 /* The length of the boundary - the buffer needs to be allocated one
    byte larger. */
 #define BOUNDARYSIZE 20
