@@ -48,6 +48,7 @@
 #include "addin-options.h"
 #include "cpphelp.h"
 #include "dispcache.h"
+#include "categorymanager.h"
 
 #include <gpg-error.h>
 #include <list>
@@ -63,8 +64,6 @@ ULONG addinLocks = 0;
 
 bool can_unload = false;
 
-static std::list<LPDISPATCH> g_ribbon_uis;
-
 static GpgolAddin * addin_instance = NULL;
 
 /* This is the main entry point for the addin
@@ -460,7 +459,7 @@ install_explorer_sinks (LPDISPATCH application)
         }
       else
         {
-          log_oom_extra ("%s:%s: created sink %p for explorer %i",
+          log_oom ("%s:%s: created sink %p for explorer %i",
                          SRCNAME, __func__, sink, i);
           GpgolAddin::get_instance ()->registerExplorerSink (sink);
         }
@@ -513,11 +512,11 @@ GpgolAddin::OnStartupComplete (SAFEARRAY** custom)
                  SRCNAME, __func__);
     }
 
-  /* Set up categories */
-  const char *decCategory = _("GpgOL: Encrypted Message");
-  const char *verifyCategory = _("GpgOL: Trusted Sender Address");
-  ensure_category_exists (m_application, decCategory, 8);
-  ensure_category_exists (m_application, verifyCategory, 5);
+  /* Clean GpgOL prefixed categories.
+     They might be left over from a crash or something unexpected
+     error. We want to avoid pollution with the signed by categories.
+  */
+  CategoryManager::removeAllGpgOLCategories ();
   install_forms ();
   m_applicationEventSink = install_ApplicationEvents_sink (m_application);
   m_explorersEventSink = install_explorer_sinks (m_application);
@@ -714,6 +713,7 @@ GpgolRibbonExtender::GetIDsOfNames (REFIID riid, LPOLESTR *rgszNames,
       ID_MAPPER (L"getIsDetailsEnabled", ID_GET_IS_DETAILS_ENABLED)
       ID_MAPPER (L"getIsCrypto", ID_GET_IS_CRYPTO_MAIL)
       ID_MAPPER (L"printDecrypted", ID_CMD_PRINT_DECRYPTED)
+      ID_MAPPER (L"openContactKey", ID_CMD_OPEN_CONTACT_KEY)
     }
 
   if (cNames > 1)
@@ -791,7 +791,7 @@ GpgolRibbonExtender::Invoke (DISPID dispid, REFIID riid, LCID lcid,
 
       case ID_ON_LOAD:
           {
-            g_ribbon_uis.push_back (parms->rgvarg[0].pdispVal);
+            GpgolAddin::get_instance ()->addRibbon (parms->rgvarg[0].pdispVal);
             return S_OK;
           }
       case ID_CMD_OPEN_OPTIONS:
@@ -803,6 +803,8 @@ GpgolRibbonExtender::Invoke (DISPID dispid, REFIID riid, LCID lcid,
         return print_decrypted (parms->rgvarg[0].pdispVal);
       case ID_GET_IS_CRYPTO_MAIL:
         return get_is_crypto_mail (parms->rgvarg[0].pdispVal, result);
+      case ID_CMD_OPEN_CONTACT_KEY:
+        return open_contact_key (parms->rgvarg[0].pdispVal);
       case ID_BTN_ENCRYPT:
       case ID_BTN_DECRYPT:
       case ID_BTN_DECRYPT_LARGE:
@@ -1069,6 +1071,43 @@ GetCustomUI_MIME (BSTR RibbonID, BSTR * RibbonXml)
         optsSTip
         );
     }
+  else if (!wcscmp (RibbonID, L"Microsoft.Outlook.Contact"))
+    {
+      gpgrt_asprintf (&buffer,
+        "<customUI xmlns=\"http://schemas.microsoft.com/office/2009/07/customui\""
+        " onLoad=\"ribbonLoaded\">"
+        " <ribbon>"
+        "   <tabs>"
+        "    <tab idMso=\"TabContact\">"
+        "     <group id=\"gpgol_contact\""
+        "            label=\"%s\">"
+        "       <button id=\"idContactAddkey\""
+        "               getImage=\"btnSignEncryptLarge\""
+        "               size=\"large\""
+        "               label=\"%s\""
+        "               screentip=\"%s\""
+        "               supertip=\"%s\""
+        "               onAction=\"openContactKey\"/>"
+        "       <dialogBoxLauncher>"
+        "         <button id=\"optsBtn_contact\""
+        "                 onAction=\"openOptions\""
+        "                 screentip=\"%s\"/>"
+        "       </dialogBoxLauncher>"
+        "     </group>"
+        "    </tab>"
+        "   </tabs>"
+        " </ribbon>"
+        "</customUI>",
+        _("GpgOL"),
+        _("OpenPGP Key"),
+        _(/* TRANSLATORS: Tooltip caption */
+          "Configure the OpenPGP key for this contact."),
+        _(/* TRANSLATORS: Tooltip content */
+          "The configured key or keys will be used for this contact even "
+          "if they are not certified."),
+        optsSTip
+        );
+    }
 
   if (buffer)
     {
@@ -1102,16 +1141,10 @@ GpgolRibbonExtender::GetCustomUI (BSTR RibbonID, BSTR * RibbonXml)
    Updated. Sadly we don't know which ribbon_ui needs updates
    so we have to invalidate everything.
 */
-void gpgoladdin_invalidate_ui ()
+void
+gpgoladdin_invalidate_ui ()
 {
-  std::list<LPDISPATCH>::iterator it;
-
-  for (it = g_ribbon_uis.begin(); it != g_ribbon_uis.end(); ++it)
-    {
-      log_debug ("%s:%s: Invalidating ribbon: %p",
-                 SRCNAME, __func__, *it);
-      invoke_oom_method (*it, "Invalidate", NULL);
-    }
+  GpgolAddin::get_instance ()->invalidateRibbons();
 }
 
 GpgolAddin *
@@ -1129,6 +1162,27 @@ GpgolAddin::get_instance ()
   return addin_instance;
 }
 
+void
+GpgolAddin::invalidateRibbons()
+{
+  /* This can only be done in the main thread so no
+     need for locking or copying here. */
+  for (const auto it: m_ribbon_uis)
+    {
+      log_debug ("%s:%s: Invalidating ribbon: %p",
+                 SRCNAME, __func__, it);
+      invoke_oom_method (it, "Invalidate", NULL);
+    }
+  log_debug ("%s:%s: Invalidation done.",
+             SRCNAME, __func__);
+}
+
+void
+GpgolAddin::addRibbon (LPDISPATCH disp)
+{
+  m_ribbon_uis.push_back (disp);
+}
+
 void
 GpgolAddin::shutdown ()
 {
@@ -1139,6 +1193,7 @@ GpgolAddin::shutdown ()
       return;
     }
 
+  m_shutdown = true;
   /* Disabling message hook */
   UnhookWindowsHookEx (m_hook);
 
@@ -1172,7 +1227,6 @@ GpgolAddin::shutdown ()
                   _("GpgOL"),
                   MB_ICONINFORMATION|MB_OK);
     }
-  m_shutdown = true;
 
   gpgol_release (m_application);
   m_application = nullptr;
@@ -1198,3 +1252,14 @@ GpgolAddin::unregisterExplorerSink (LPDISPATCH sink)
   log_error ("%s:%s: Unregister %p which was not registered.",
              SRCNAME, __func__, sink);
 }
+
+std::shared_ptr <CategoryManager>
+GpgolAddin::get_category_mngr ()
+{
+  if (!m_category_mngr)
+    {
+      m_category_mngr = std::shared_ptr<CategoryManager> (
+                                                  new CategoryManager ());
+    }
+  return m_category_mngr;
+}
