@@ -28,6 +28,8 @@
 #include <gpg-error.h>
 #include <gpgme++/context.h>
 #include <gpgme++/key.h>
+#include <gpgme++/data.h>
+#include <gpgme++/importresult.h>
 
 #include <windows.h>
 
@@ -38,6 +40,7 @@
 GPGRT_LOCK_DEFINE (keycache_lock);
 GPGRT_LOCK_DEFINE (fpr_map_lock);
 GPGRT_LOCK_DEFINE (update_lock);
+GPGRT_LOCK_DEFINE (import_lock);
 static KeyCache* singleton = nullptr;
 
 /** At some point we need to set a limit. There
@@ -62,6 +65,7 @@ namespace
           m_mbox (mbox),
           m_mail (mail)
         {
+          TSTART;
           s_thread_cnt++;
           Mail::lockDelete ();
           if (Mail::isValidPtr (m_mail))
@@ -69,10 +73,12 @@ namespace
               m_mail->incrementLocateCount ();
             }
           Mail::unlockDelete ();
+          TRETURN;
         };
 
         ~LocateArgs()
         {
+          TSTART;
           s_thread_cnt--;
           Mail::lockDelete ();
           if (Mail::isValidPtr (m_mail))
@@ -80,6 +86,7 @@ namespace
               m_mail->decrementLocateCount ();
             }
           Mail::unlockDelete ();
+          TRETURN;
         }
 
         std::string m_mbox;
@@ -89,14 +96,17 @@ namespace
 
 typedef std::pair<std::string, GpgME::Protocol> update_arg_t;
 
+typedef std::pair<std::unique_ptr<LocateArgs>, std::string> import_arg_t;
+
 static DWORD WINAPI
 do_update (LPVOID arg)
 {
+  TSTART;
   auto args = std::unique_ptr<update_arg_t> ((update_arg_t*) arg);
 
-  log_mime_parser ("%s:%s updating: \"%s\" with protocol %s",
-                   SRCNAME, __func__, args->first.c_str (),
-                   to_cstr (args->second));
+  log_debug ("%s:%s updating: \"%s\" with protocol %s",
+             SRCNAME, __func__, anonstr (args->first.c_str ()),
+             to_cstr (args->second));
 
   auto ctx = std::unique_ptr<GpgME::Context> (GpgME::Context::createForProtocol
                                               (args->second));
@@ -106,7 +116,7 @@ do_update (LPVOID arg)
       TRACEPOINT;
       KeyCache::instance ()->onUpdateJobDone (args->first.c_str(),
                                               GpgME::Key ());
-      return 0;
+      TRETURN 0;
     }
 
   ctx->setKeyListMode (GpgME::KeyListMode::Local |
@@ -120,20 +130,92 @@ do_update (LPVOID arg)
   if (newKey.isNull())
     {
       log_debug ("%s:%s Failed to find key for %s",
-                 SRCNAME, __func__, args->first.c_str ());
+                 SRCNAME, __func__, anonstr (args->first.c_str ()));
     }
   if (err)
     {
-      log_debug ("%s:%s Failed to find key for %s err: ",
-                 SRCNAME, __func__, err.asString ());
+      log_debug ("%s:%s Failed to find key for %s err: %s",
+                 SRCNAME, __func__, anonstr (args->first.c_str()),
+                 err.asString ());
     }
   KeyCache::instance ()->onUpdateJobDone (args->first.c_str(),
                                           newKey);
   log_debug ("%s:%s Update job done",
              SRCNAME, __func__);
-  return 0;
+  TRETURN 0;
 }
 
+static DWORD WINAPI
+do_import (LPVOID arg)
+{
+  TSTART;
+  auto args = std::unique_ptr<import_arg_t> ((import_arg_t*) arg);
+
+  const std::string mbox = args->first->m_mbox;
+
+  log_debug ("%s:%s importing for: \"%s\" with data \n%s",
+             SRCNAME, __func__, anonstr (mbox.c_str ()),
+             anonstr (args->second.c_str ()));
+  auto ctx = std::unique_ptr<GpgME::Context> (GpgME::Context::createForProtocol
+                                              (GpgME::OpenPGP));
+
+  if (!ctx)
+    {
+      TRACEPOINT;
+      TRETURN 0;
+    }
+  // We want to avoid unneccessary copies. The c_str will be valid
+  // until args goes out of scope.
+  const char *keyStr = args->second.c_str ();
+  GpgME::Data data (keyStr, strlen (keyStr), /* copy */ false);
+
+  if (data.type () != GpgME::Data::PGPKey)
+    {
+      log_debug ("%s:%s Data for: %s is not a PGP Key",
+                 SRCNAME, __func__, anonstr (mbox.c_str ()));
+      TRETURN 0;
+    }
+  data.rewind ();
+
+  const auto result = ctx->importKeys (data);
+
+  std::vector<std::string> fingerprints;
+  for (const auto import: result.imports())
+    {
+      if (import.error())
+        {
+          log_debug ("%s:%s Error importing: %s",
+                     SRCNAME, __func__, import.error().asString());
+          continue;
+        }
+      const char *fpr = import.fingerprint ();
+      if (!fpr)
+        {
+          TRACEPOINT;
+          continue;
+        }
+
+      update_arg_t * update_args = new update_arg_t;
+      update_args->first = std::string (fpr);
+      update_args->second = GpgME::OpenPGP;
+
+      // We do it blocking to be sure that when all imports
+      // are done they are also part of the keycache.
+      do_update ((LPVOID) update_args);
+
+      fingerprints.push_back (fpr);
+      log_debug ("%s:%s Imported: %s from addressbook.",
+                 SRCNAME, __func__, anonstr (fpr));
+    }
+
+  KeyCache::instance ()->onAddrBookImportJobDone (mbox, fingerprints);
+
+  log_debug ("%s:%s Import job done for: %s",
+             SRCNAME, __func__, anonstr (mbox.c_str ()));
+  TRETURN 0;
+}
+
+
 class KeyCache::Private
 {
 public:
@@ -144,7 +226,8 @@ public:
 
   void setPgpKey(const std::string &mbox, const GpgME::Key &key)
   {
-    gpgrt_lock_lock (&keycache_lock);
+    TSTART;
+    gpgol_lock (&keycache_lock);
     auto it = m_pgp_key_map.find (mbox);
 
     if (it == m_pgp_key_map.end ())
@@ -156,12 +239,14 @@ public:
         it->second = key;
       }
     insertOrUpdateInFprMap (key);
-    gpgrt_lock_unlock (&keycache_lock);
+    gpgol_unlock (&keycache_lock);
+    TRETURN;
   }
 
   void setSmimeKey(const std::string &mbox, const GpgME::Key &key)
   {
-    gpgrt_lock_lock (&keycache_lock);
+    TSTART;
+    gpgol_lock (&keycache_lock);
     auto it = m_smime_key_map.find (mbox);
 
     if (it == m_smime_key_map.end ())
@@ -173,12 +258,14 @@ public:
         it->second = key;
       }
     insertOrUpdateInFprMap (key);
-    gpgrt_lock_unlock (&keycache_lock);
+    gpgol_unlock (&keycache_lock);
+    TRETURN;
   }
 
   void setPgpKeySecret(const std::string &mbox, const GpgME::Key &key)
   {
-    gpgrt_lock_lock (&keycache_lock);
+    TSTART;
+    gpgol_lock (&keycache_lock);
     auto it = m_pgp_skey_map.find (mbox);
 
     if (it == m_pgp_skey_map.end ())
@@ -190,12 +277,14 @@ public:
         it->second = key;
       }
     insertOrUpdateInFprMap (key);
-    gpgrt_lock_unlock (&keycache_lock);
+    gpgol_unlock (&keycache_lock);
+    TRETURN;
   }
 
   void setSmimeKeySecret(const std::string &mbox, const GpgME::Key &key)
   {
-    gpgrt_lock_lock (&keycache_lock);
+    TSTART;
+    gpgol_lock (&keycache_lock);
     auto it = m_smime_skey_map.find (mbox);
 
     if (it == m_smime_skey_map.end ())
@@ -207,146 +296,197 @@ public:
         it->second = key;
       }
     insertOrUpdateInFprMap (key);
-    gpgrt_lock_unlock (&keycache_lock);
+    gpgol_unlock (&keycache_lock);
+    TRETURN;
+  }
+
+  std::vector<GpgME::Key> getPGPOverrides (const char *addr)
+  {
+    TSTART;
+    std::vector<GpgME::Key> ret;
+
+    if (!addr)
+      {
+        TRETURN ret;
+      }
+    auto mbox = GpgME::UserID::addrSpecFromString (addr);
+
+    gpgol_lock (&keycache_lock);
+    const auto it = m_addr_book_overrides.find (mbox);
+    if (it == m_addr_book_overrides.end ())
+      {
+        gpgol_unlock (&keycache_lock);
+        TRETURN ret;
+      }
+    for (const auto fpr: it->second)
+      {
+        const auto key = getByFpr (fpr.c_str (), false);
+        if (key.isNull())
+          {
+            log_debug ("%s:%s: No key for %s in the cache?!",
+                       SRCNAME, __func__, anonstr (fpr.c_str()));
+            continue;
+          }
+        ret.push_back (key);
+      }
+
+    gpgol_unlock (&keycache_lock);
+    TRETURN ret;
   }
 
   GpgME::Key getKey (const char *addr, GpgME::Protocol proto)
   {
+    TSTART;
     if (!addr)
       {
-        return GpgME::Key();
+        TRETURN GpgME::Key();
       }
     auto mbox = GpgME::UserID::addrSpecFromString (addr);
 
     if (proto == GpgME::OpenPGP)
       {
-        gpgrt_lock_lock (&keycache_lock);
+        gpgol_lock (&keycache_lock);
         const auto it = m_pgp_key_map.find (mbox);
 
         if (it == m_pgp_key_map.end ())
           {
-            gpgrt_lock_unlock (&keycache_lock);
-            return GpgME::Key();
+            gpgol_unlock (&keycache_lock);
+            TRETURN GpgME::Key();
           }
         const auto ret = it->second;
-        gpgrt_lock_unlock (&keycache_lock);
+        gpgol_unlock (&keycache_lock);
 
-        return ret;
+        TRETURN ret;
       }
-    gpgrt_lock_lock (&keycache_lock);
+    gpgol_lock (&keycache_lock);
     const auto it = m_smime_key_map.find (mbox);
 
     if (it == m_smime_key_map.end ())
       {
-        gpgrt_lock_unlock (&keycache_lock);
-        return GpgME::Key();
+        gpgol_unlock (&keycache_lock);
+        TRETURN GpgME::Key();
       }
     const auto ret = it->second;
-    gpgrt_lock_unlock (&keycache_lock);
+    gpgol_unlock (&keycache_lock);
 
-    return ret;
+    TRETURN ret;
   }
 
   GpgME::Key getSKey (const char *addr, GpgME::Protocol proto)
   {
+    TSTART;
     if (!addr)
       {
-        return GpgME::Key();
+        TRETURN GpgME::Key();
       }
     auto mbox = GpgME::UserID::addrSpecFromString (addr);
 
     if (proto == GpgME::OpenPGP)
       {
-        gpgrt_lock_lock (&keycache_lock);
+        gpgol_lock (&keycache_lock);
         const auto it = m_pgp_skey_map.find (mbox);
 
         if (it == m_pgp_skey_map.end ())
           {
-            gpgrt_lock_unlock (&keycache_lock);
-            return GpgME::Key();
+            gpgol_unlock (&keycache_lock);
+            TRETURN GpgME::Key();
           }
         const auto ret = it->second;
-        gpgrt_lock_unlock (&keycache_lock);
+        gpgol_unlock (&keycache_lock);
 
-        return ret;
+        TRETURN ret;
       }
-    gpgrt_lock_lock (&keycache_lock);
+    gpgol_lock (&keycache_lock);
     const auto it = m_smime_skey_map.find (mbox);
 
     if (it == m_smime_skey_map.end ())
       {
-        gpgrt_lock_unlock (&keycache_lock);
-        return GpgME::Key();
+        gpgol_unlock (&keycache_lock);
+        TRETURN GpgME::Key();
       }
     const auto ret = it->second;
-    gpgrt_lock_unlock (&keycache_lock);
+    gpgol_unlock (&keycache_lock);
 
-    return ret;
+    TRETURN ret;
   }
 
   GpgME::Key getSigningKey (const char *addr, GpgME::Protocol proto)
   {
+    TSTART;
     const auto key = getSKey (addr, proto);
     if (key.isNull())
       {
-        log_mime_parser ("%s:%s: secret key for %s is null",
-                   SRCNAME, __func__, addr);
-        return key;
+        log_debug ("%s:%s: secret key for %s is null",
+                   SRCNAME, __func__, anonstr (addr));
+        TRETURN key;
       }
     if (!key.canReallySign())
       {
-        log_mime_parser ("%s:%s: Discarding key for %s because it can't sign",
-                   SRCNAME, __func__, addr);
-        return GpgME::Key();
+        log_debug ("%s:%s: Discarding key for %s because it can't sign",
+                   SRCNAME, __func__, anonstr (addr));
+        TRETURN GpgME::Key();
       }
     if (!key.hasSecret())
       {
-        log_mime_parser ("%s:%s: Discarding key for %s because it has no secret",
-                   SRCNAME, __func__, addr);
-        return GpgME::Key();
+        log_debug ("%s:%s: Discarding key for %s because it has no secret",
+                   SRCNAME, __func__, anonstr (addr));
+        TRETURN GpgME::Key();
       }
     if (in_de_vs_mode () && !key.isDeVs())
       {
-        log_mime_parser ("%s:%s: signing key for %s is not deVS",
-                   SRCNAME, __func__, addr);
-        return GpgME::Key();
+        log_debug ("%s:%s: signing key for %s is not deVS",
+                   SRCNAME, __func__, anonstr (addr));
+        TRETURN GpgME::Key();
       }
-    return key;
+    TRETURN key;
   }
 
   std::vector<GpgME::Key> getEncryptionKeys (const std::vector<std::string>
                                              &recipients,
                                              GpgME::Protocol proto)
   {
+    TSTART;
     std::vector<GpgME::Key> ret;
     if (recipients.empty ())
       {
         TRACEPOINT;
-        return ret;
+        TRETURN ret;
       }
     for (const auto &recip: recipients)
       {
+        if (proto == GpgME::OpenPGP)
+          {
+            const auto overrides = getPGPOverrides (recip.c_str ());
+
+            if (!overrides.empty())
+              {
+                ret.insert (ret.end (), overrides.begin (), overrides.end ());
+                log_debug ("%s:%s: Using overides for %s",
+                           SRCNAME, __func__, anonstr (recip.c_str ()));
+                continue;
+              }
+          }
         const auto key = getKey (recip.c_str (), proto);
         if (key.isNull())
           {
-            log_mime_parser ("%s:%s: No key for %s. no internal encryption",
-                       SRCNAME, __func__, recip.c_str ());
-            return std::vector<GpgME::Key>();
+            log_debug ("%s:%s: No key for %s. no internal encryption",
+                       SRCNAME, __func__, anonstr (recip.c_str ()));
+            TRETURN std::vector<GpgME::Key>();
           }
 
         if (!key.canEncrypt() || key.isRevoked() ||
             key.isExpired() || key.isDisabled() || key.isInvalid())
           {
-            log_mime_parser ("%s:%s: Invalid key for %s. no internal encryption",
-                       SRCNAME, __func__, recip.c_str ());
-            return std::vector<GpgME::Key>();
+            log_data ("%s:%s: Invalid key for %s. no internal encryption",
+                       SRCNAME, __func__, anonstr (recip.c_str ()));
+            TRETURN std::vector<GpgME::Key>();
           }
 
         if (in_de_vs_mode () && !key.isDeVs ())
           {
-            log_mime_parser ("%s:%s: key for %s is not deVS",
-                       SRCNAME, __func__, recip.c_str ());
-            return std::vector<GpgME::Key>();
+            log_data ("%s:%s: key for %s is not deVS",
+                      SRCNAME, __func__, anonstr (recip.c_str ()));
+            TRETURN std::vector<GpgME::Key>();
           }
 
         bool validEnough = false;
@@ -368,24 +508,25 @@ public:
           }
         if (!validEnough)
           {
-            log_mime_parser ("%s:%s: UID for %s does not have at least marginal trust",
-                             SRCNAME, __func__, recip.c_str ());
-            return std::vector<GpgME::Key>();
+            log_debug ("%s:%s: UID for %s does not have at least marginal trust",
+                       SRCNAME, __func__, anonstr (recip.c_str ()));
+            TRETURN std::vector<GpgME::Key>();
           }
         // Accepting key
         ret.push_back (key);
       }
-    return ret;
+    TRETURN ret;
   }
 
   void insertOrUpdateInFprMap (const GpgME::Key &key)
     {
+      TSTART;
       if (key.isNull() || !key.primaryFingerprint())
         {
           TRACEPOINT;
-          return;
+          TRETURN;
         }
-      gpgrt_lock_lock (&fpr_map_lock);
+      gpgol_lock (&fpr_map_lock);
 
       /* First ensure that we have the subkeys mapped to the primary
          fpr */
@@ -405,14 +546,14 @@ public:
 
       auto it = m_fpr_map.find (primaryFpr);
 
-      log_mime_parser ("%s:%s \"%s\" updated.",
-                       SRCNAME, __func__, primaryFpr);
+      log_debug ("%s:%s \"%s\" updated.",
+                 SRCNAME, __func__, anonstr (primaryFpr));
       if (it == m_fpr_map.end ())
         {
           m_fpr_map.insert (std::make_pair (primaryFpr, key));
 
-          gpgrt_lock_unlock (&fpr_map_lock);
-          return;
+          gpgol_unlock (&fpr_map_lock);
+          TRETURN;
         }
 
       if (it->second.hasSecret () && !key.hasSecret())
@@ -427,25 +568,27 @@ public:
         {
           it->second = key;
         }
-      gpgrt_lock_unlock (&fpr_map_lock);
-      return;
+      gpgol_unlock (&fpr_map_lock);
+      TRETURN;
     }
 
   GpgME::Key getFromMap (const char *fpr) const
   {
+    TSTART;
     if (!fpr)
       {
         TRACEPOINT;
-        return GpgME::Key();
+        TRETURN GpgME::Key();
       }
 
-    gpgrt_lock_lock (&fpr_map_lock);
+    gpgol_lock (&fpr_map_lock);
     std::string primaryFpr;
     const auto it = m_sub_fpr_map.find (fpr);
     if (it != m_sub_fpr_map.end ())
       {
         log_debug ("%s:%s using \"%s\" for \"%s\"",
-                   SRCNAME, __func__, it->second.c_str(), fpr);
+                   SRCNAME, __func__, anonstr (it->second.c_str()),
+                   anonstr (fpr));
         primaryFpr = it->second;
       }
     else
@@ -457,19 +600,20 @@ public:
     if (keyIt != m_fpr_map.end ())
       {
         const auto ret = keyIt->second;
-        gpgrt_lock_unlock (&fpr_map_lock);
-        return ret;
+        gpgol_unlock (&fpr_map_lock);
+        TRETURN ret;
       }
-    gpgrt_lock_unlock (&fpr_map_lock);
-    return GpgME::Key();
+    gpgol_unlock (&fpr_map_lock);
+    TRETURN GpgME::Key();
   }
 
   GpgME::Key getByFpr (const char *fpr, bool block) const
     {
+      TSTART;
       if (!fpr)
         {
           TRACEPOINT;
-          return GpgME::Key ();
+          TRETURN GpgME::Key ();
         }
 
       TRACEPOINT;
@@ -483,100 +627,170 @@ public:
               const std::string sFpr (fpr);
               int i = 0;
 
-              gpgrt_lock_lock (&update_lock);
+              gpgol_lock (&update_lock);
               while (m_update_jobs.find(sFpr) != m_update_jobs.end ())
                 {
                   i++;
                   if (i % 100 == 0)
                     {
                       log_debug ("%s:%s Waiting on update for \"%s\"",
-                                 SRCNAME, __func__, fpr);
+                                 SRCNAME, __func__, anonstr (fpr));
                     }
-                  gpgrt_lock_unlock (&update_lock);
+                  gpgol_unlock (&update_lock);
                   Sleep (10);
-                  gpgrt_lock_lock (&update_lock);
+                  gpgol_lock (&update_lock);
                   if (i == 3000)
                     {
                       /* Just to be on the save side */
                       log_error ("%s:%s Waiting on update for \"%s\" "
                                  "failed! Bug!",
-                                 SRCNAME, __func__, fpr);
+                                 SRCNAME, __func__, anonstr (fpr));
                       break;
                     }
                 }
-              gpgrt_lock_unlock (&update_lock);
+              gpgol_unlock (&update_lock);
 
               TRACEPOINT;
               const auto ret2 = getFromMap (fpr);
               if (ret2.isNull ())
                 {
                   log_debug ("%s:%s Cache miss after blocking check %s.",
-                             SRCNAME, __func__, fpr);
+                             SRCNAME, __func__, anonstr (fpr));
                 }
               else
                 {
                   log_debug ("%s:%s Cache hit after wait for %s.",
-                             SRCNAME, __func__, fpr);
-                  return ret2;
+                             SRCNAME, __func__, anonstr (fpr));
+                  TRETURN ret2;
                 }
             }
           log_debug ("%s:%s Cache miss for %s.",
-                     SRCNAME, __func__, fpr);
-          return GpgME::Key();
+                     SRCNAME, __func__, anonstr (fpr));
+          TRETURN GpgME::Key();
         }
 
       log_debug ("%s:%s Cache hit for %s.",
-                 SRCNAME, __func__, fpr);
-      return ret;
+                 SRCNAME, __func__, anonstr (fpr));
+      TRETURN ret;
     }
 
   void update (const char *fpr, GpgME::Protocol proto)
      {
+       TSTART;
        if (!fpr)
          {
-           return;
+           TRETURN;
          }
        const std::string sFpr (fpr);
-       gpgrt_lock_lock (&update_lock);
+       gpgol_lock (&update_lock);
        if (m_update_jobs.find(sFpr) != m_update_jobs.end ())
          {
            log_debug ("%s:%s Update for \"%s\" already in progress.",
-                      SRCNAME, __func__, fpr);
-           gpgrt_lock_unlock (&update_lock);
+                      SRCNAME, __func__, anonstr (fpr));
+           gpgol_unlock (&update_lock);
          }
 
        m_update_jobs.insert (sFpr);
-       gpgrt_lock_unlock (&update_lock);
+       gpgol_unlock (&update_lock);
        update_arg_t * args = new update_arg_t;
        args->first = sFpr;
        args->second = proto;
        CloseHandle (CreateThread (NULL, 0, do_update,
                                   (LPVOID) args, 0,
                                   NULL));
+       TRETURN;
      }
 
   void onUpdateJobDone (const char *fpr, const GpgME::Key &key)
     {
+      TSTART;
       if (!fpr)
         {
-          return;
+          TRETURN;
         }
       TRACEPOINT;
       insertOrUpdateInFprMap (key);
-      gpgrt_lock_lock (&update_lock);
+      gpgol_lock (&update_lock);
       const auto it = m_update_jobs.find(fpr);
 
       if (it == m_update_jobs.end())
         {
-          log_error ("%s:%s Update for \"%s\" already finished.",
-                     SRCNAME, __func__, fpr);
-          gpgrt_lock_unlock (&update_lock);
-          return;
+          log_debug ("%s:%s Update for \"%s\" already finished.",
+                     SRCNAME, __func__, anonstr (fpr));
+          gpgol_unlock (&update_lock);
+          TRETURN;
         }
       m_update_jobs.erase (it);
-      gpgrt_lock_unlock (&update_lock);
+      gpgol_unlock (&update_lock);
       TRACEPOINT;
-      return;
+      TRETURN;
+    }
+
+  void importFromAddrBook (const std::string &mbox, const char *data,
+                           Mail *mail)
+    {
+      TSTART;
+      if (!data || mbox.empty() || !mail)
+        {
+          TRACEPOINT;
+          TRETURN;
+        }
+
+      std::string sdata (data);
+      trim (sdata);
+      if (sdata.empty())
+        {
+          TRETURN;
+        }
+      gpgol_lock (&import_lock);
+      if (m_import_jobs.find (mbox) != m_import_jobs.end ())
+        {
+          log_debug ("%s:%s import for \"%s\" already in progress.",
+                     SRCNAME, __func__, anonstr (mbox.c_str ()));
+          gpgol_unlock (&import_lock);
+        }
+      m_import_jobs.insert (mbox);
+      gpgol_unlock (&import_lock);
+
+      import_arg_t * args = new import_arg_t;
+      args->first = std::unique_ptr<LocateArgs> (new LocateArgs (mbox, mail));
+      args->second = sdata;
+      CloseHandle (CreateThread (NULL, 0, do_import,
+                                 (LPVOID) args, 0,
+                                 NULL));
+
+      TRETURN;
+    }
+
+  void onAddrBookImportJobDone (const std::string &mbox,
+                                const std::vector<std::string> &result_fprs)
+    {
+      TSTART;
+      gpgol_lock (&keycache_lock);
+      auto it = m_addr_book_overrides.find (mbox);
+      if (it != m_addr_book_overrides.end ())
+        {
+          it->second = result_fprs;
+        }
+      else
+        {
+          m_addr_book_overrides.insert (
+                std::make_pair (mbox, result_fprs));
+        }
+      gpgol_unlock (&keycache_lock);
+      gpgol_lock (&import_lock);
+      const auto job_it = m_import_jobs.find(mbox);
+
+      if (job_it == m_import_jobs.end())
+        {
+          log_error ("%s:%s import for \"%s\" already finished.",
+                     SRCNAME, __func__, anonstr (mbox.c_str ()));
+          gpgol_unlock (&import_lock);
+          TRETURN;
+        }
+      m_import_jobs.erase (job_it);
+      gpgol_unlock (&import_lock);
+      TRETURN;
     }
 
   std::unordered_map<std::string, GpgME::Key> m_pgp_key_map;
@@ -585,7 +799,10 @@ public:
   std::unordered_map<std::string, GpgME::Key> m_smime_skey_map;
   std::unordered_map<std::string, GpgME::Key> m_fpr_map;
   std::unordered_map<std::string, std::string> m_sub_fpr_map;
+  std::unordered_map<std::string, std::vector<std::string> >
+    m_addr_book_overrides;
   std::set<std::string> m_update_jobs;
+  std::set<std::string> m_import_jobs;
 };
 
 KeyCache::KeyCache():
@@ -615,112 +832,205 @@ KeyCache::getEncryptionKeys (const std::vector<std::string> &recipients, GpgME::
   return d->getEncryptionKeys (recipients, proto);
 }
 
+static GpgME::Key
+get_most_valid_key_simple (const std::vector<GpgME::Key> &keys)
+{
+  GpgME::Key candidate;
+  for (const auto &key: keys)
+    {
+      if (key.isRevoked() || key.isExpired() ||
+          key.isDisabled() || key.isInvalid())
+        {
+          log_debug ("%s:%s: Skipping invalid S/MIME key",
+                     SRCNAME, __func__);
+          continue;
+        }
+      if (candidate.isNull() || !candidate.numUserIDs())
+        {
+          if (key.numUserIDs() &&
+              candidate.userID(0).validity() <= key.userID(0).validity())
+            {
+              candidate = key;
+            }
+        }
+    }
+  return candidate;
+}
+
+static std::vector<GpgME::Key>
+get_local_smime_keys (const std::string &addr)
+{
+  TSTART;
+  std::vector<GpgME::Key> keys;
+  auto ctx = std::unique_ptr<GpgME::Context> (
+                                              GpgME::Context::createForProtocol (GpgME::CMS));
+  if (!ctx)
+    {
+      TRACEPOINT;
+      TRETURN keys;
+    }
+  // We need to validate here to fetch CRL's
+  ctx->setKeyListMode (GpgME::KeyListMode::Local |
+                       GpgME::KeyListMode::Validate |
+                       GpgME::KeyListMode::Signatures);
+  GpgME::Error e = ctx->startKeyListing (addr.c_str());
+  if (e)
+    {
+      TRACEPOINT;
+      TRETURN keys;
+    }
+
+  GpgME::Error err;
+  do {
+      keys.push_back(ctx->nextKey(err));
+  } while (!err);
+  keys.pop_back();
+
+  TRETURN keys;
+}
+
+static std::vector<GpgME::Key>
+get_extern_smime_keys (const std::string &addr, bool import)
+{
+  TSTART;
+  std::vector<GpgME::Key> keys;
+  auto ctx = std::unique_ptr<GpgME::Context> (
+                                              GpgME::Context::createForProtocol (GpgME::CMS));
+  if (!ctx)
+    {
+      TRACEPOINT;
+      TRETURN keys;
+    }
+  // We need to validate here to fetch CRL's
+  ctx->setKeyListMode (GpgME::KeyListMode::Extern);
+  GpgME::Error e = ctx->startKeyListing (addr.c_str());
+  if (e)
+    {
+      TRACEPOINT;
+      TRETURN keys;
+    }
+
+  GpgME::Error err;
+  do
+    {
+      const auto key = ctx->nextKey (err);
+      if (!err && !key.isNull())
+        {
+          keys.push_back (key);
+          log_debug ("%s:%s: Found extern S/MIME key for %s with fpr: %s",
+                     SRCNAME, __func__, anonstr (addr.c_str()),
+                     anonstr (key.primaryFingerprint()));
+        }
+    } while (!err);
+
+  if (import && keys.size ())
+    {
+      const GpgME::ImportResult res = ctx->importKeys(keys);
+      log_debug ("%s:%s: Import result for %s: err: %s",
+                 SRCNAME, __func__, anonstr (addr.c_str()),
+                 res.error ().asString ());
+
+    }
+
+  TRETURN keys;
+}
+
 static DWORD WINAPI
 do_locate (LPVOID arg)
 {
+  TSTART;
   if (!arg)
     {
-      return 0;
+      TRETURN 0;
     }
 
   auto args = std::unique_ptr<LocateArgs> ((LocateArgs *) arg);
 
   const auto addr = args->m_mbox;
 
-  log_mime_parser ("%s:%s searching key for addr: \"%s\"",
-                   SRCNAME, __func__, addr.c_str());
+  log_debug ("%s:%s searching key for addr: \"%s\"",
+             SRCNAME, __func__, anonstr (addr.c_str()));
 
   const auto k = GpgME::Key::locate (addr.c_str());
 
   if (!k.isNull ())
     {
-      log_mime_parser ("%s:%s found key for addr: \"%s\":%s",
-                       SRCNAME, __func__, addr.c_str(),
-                       k.primaryFingerprint());
+      log_debug ("%s:%s found key for addr: \"%s\":%s",
+                 SRCNAME, __func__, anonstr (addr.c_str()),
+                 anonstr (k.primaryFingerprint()));
       KeyCache::instance ()->setPgpKey (addr, k);
     }
+  log_debug ("%s:%s pgp locate done",
+             SRCNAME, __func__);
 
   if (opt.enable_smime)
     {
-      auto ctx = std::unique_ptr<GpgME::Context> (
-                          GpgME::Context::createForProtocol (GpgME::CMS));
-      if (!ctx)
-        {
-          TRACEPOINT;
-          return 0;
-        }
-      // We need to validate here to fetch CRL's
-      ctx->setKeyListMode (GpgME::KeyListMode::Local |
-                           GpgME::KeyListMode::Validate |
-                           GpgME::KeyListMode::Signatures);
-      GpgME::Error e = ctx->startKeyListing (addr.c_str());
-      if (e)
-        {
-          TRACEPOINT;
-          return 0;
-        }
-
-      std::vector<GpgME::Key> keys;
-      GpgME::Error err;
-      do {
-          keys.push_back(ctx->nextKey(err));
-      } while (!err);
-      keys.pop_back();
-
-      GpgME::Key candidate;
-      for (const auto &key: keys)
-        {
-          if (key.isRevoked() || key.isExpired() ||
-              key.isDisabled() || key.isInvalid())
-            {
-              log_mime_parser ("%s:%s: Skipping invalid S/MIME key",
-                               SRCNAME, __func__);
-              continue;
-            }
-          if (candidate.isNull() || !candidate.numUserIDs())
-            {
-              if (key.numUserIDs() &&
-                  candidate.userID(0).validity() <= key.userID(0).validity())
-                {
-                  candidate = key;
-                }
-            }
-        }
+      GpgME::Key candidate = get_most_valid_key_simple (
+                                    get_local_smime_keys (addr));
       if (!candidate.isNull())
         {
-          log_mime_parser ("%s:%s found SMIME key for addr: \"%s\":%s",
-                           SRCNAME, __func__, addr.c_str(),
-                           candidate.primaryFingerprint());
+          log_debug ("%s:%s found SMIME key for addr: \"%s\":%s",
+                     SRCNAME, __func__, anonstr (addr.c_str()),
+                     anonstr (candidate.primaryFingerprint()));
           KeyCache::instance()->setSmimeKey (addr, candidate);
+          TRETURN 0;
+        }
+      if (!opt.search_smime_servers || (!k.isNull() && !opt.prefer_smime))
+        {
+          log_debug ("%s:%s Found no S/MIME key locally and external "
+                     "search is disabled.", SRCNAME, __func__);
+          TRETURN 0;
+        }
+      /* Search for extern keys and import them */
+      const auto externs = get_extern_smime_keys (addr, true);
+      if (externs.empty())
+        {
+          TRETURN 0;
+        }
+      /* We found and imported external keys. We need to get them
+         locally now to ensure that they are valid etc. */
+      candidate = get_most_valid_key_simple (
+                                    get_local_smime_keys (addr));
+      if (!candidate.isNull())
+        {
+          log_debug ("%s:%s found ext. SMIME key for addr: \"%s\":%s",
+                     SRCNAME, __func__, anonstr (addr.c_str()),
+                     anonstr (candidate.primaryFingerprint()));
+          KeyCache::instance()->setSmimeKey (addr, candidate);
+          TRETURN 0;
+        }
+      else
+        {
+          log_debug ("%s:%s: Found no valid key in extern S/MIME certs",
+                     SRCNAME, __func__);
         }
     }
-  log_debug ("%s:%s locator thread done",
-             SRCNAME, __func__);
-  return 0;
+  TRETURN 0;
 }
 
 static void
 locate_secret (const char *addr, GpgME::Protocol proto)
 {
+  TSTART;
   auto ctx = std::unique_ptr<GpgME::Context> (
                       GpgME::Context::createForProtocol (proto));
   if (!ctx)
     {
       TRACEPOINT;
-      return;
+      TRETURN;
     }
   if (!addr)
     {
       TRACEPOINT;
-      return;
+      TRETURN;
     }
   const auto mbox = GpgME::UserID::addrSpecFromString (addr);
 
   if (mbox.empty())
     {
       log_debug ("%s:%s: Empty mbox for addr %s",
-                 SRCNAME, __func__, addr);
-      return;
+                 SRCNAME, __func__, anonstr (addr));
+      TRETURN;
     }
 
   // We need to validate here to fetch CRL's
@@ -730,7 +1040,7 @@ locate_secret (const char *addr, GpgME::Protocol proto)
   if (e)
     {
       TRACEPOINT;
-      return;
+      TRETURN;
     }
 
   std::vector<GpgME::Key> keys;
@@ -745,42 +1055,43 @@ locate_secret (const char *addr, GpgME::Protocol proto)
       if (key.isRevoked() || key.isExpired() ||
           key.isDisabled() || key.isInvalid())
         {
-          if ((opt.enable_debug & DBG_MIME_PARSER))
+          if ((opt.enable_debug & DBG_DATA))
             {
               std::stringstream ss;
               ss << key;
-              log_mime_parser ("%s:%s: Skipping invalid secret key %s",
-                               SRCNAME, __func__, ss.str().c_str());
+              log_data ("%s:%s: Skipping invalid secret key %s",
+                        SRCNAME, __func__, ss.str().c_str());
             }
           continue;
         }
       if (proto == GpgME::OpenPGP)
         {
-          log_mime_parser ("%s:%s found pgp skey for addr: \"%s\":%s",
-                           SRCNAME, __func__, mbox.c_str(),
-                           key.primaryFingerprint());
+          log_debug ("%s:%s found pgp skey for addr: \"%s\":%s",
+                     SRCNAME, __func__, anonstr (mbox.c_str()),
+                     anonstr (key.primaryFingerprint()));
           KeyCache::instance()->setPgpKeySecret (mbox, key);
-          return;
+          TRETURN;
         }
       if (proto == GpgME::CMS)
         {
-          log_mime_parser ("%s:%s found cms skey for addr: \"%s\":%s",
-                           SRCNAME, __func__, mbox.c_str (),
-                           key.primaryFingerprint());
+          log_debug ("%s:%s found cms skey for addr: \"%s\":%s",
+                     SRCNAME, __func__, anonstr (mbox.c_str ()),
+                     anonstr (key.primaryFingerprint()));
           KeyCache::instance()->setSmimeKeySecret (mbox, key);
-          return;
+          TRETURN;
         }
     } while (!err);
-  return;
+  TRETURN;
 }
 
 static DWORD WINAPI
 do_locate_secret (LPVOID arg)
 {
+  TSTART;
   auto args = std::unique_ptr<LocateArgs> ((LocateArgs *) arg);
 
-  log_mime_parser ("%s:%s searching secret key for addr: \"%s\"",
-                   SRCNAME, __func__, args->m_mbox.c_str ());
+  log_debug ("%s:%s searching secret key for addr: \"%s\"",
+             SRCNAME, __func__, anonstr (args->m_mbox.c_str ()));
 
   locate_secret (args->m_mbox.c_str(), GpgME::OpenPGP);
   if (opt.enable_smime)
@@ -789,7 +1100,7 @@ do_locate_secret (LPVOID arg)
     }
   log_debug ("%s:%s locator sthread thread done",
              SRCNAME, __func__);
-  return 0;
+  TRETURN 0;
 }
 
 void
@@ -804,17 +1115,18 @@ KeyCache::startLocate (const std::vector<std::string> &addrs, Mail *mail) const
 void
 KeyCache::startLocate (const char *addr, Mail *mail) const
 {
+  TSTART;
   if (!addr)
     {
       TRACEPOINT;
-      return;
+      TRETURN;
     }
   std::string recp = GpgME::UserID::addrSpecFromString (addr);
   if (recp.empty ())
     {
-      return;
+      TRETURN;
     }
-  gpgrt_lock_lock (&keycache_lock);
+  gpgol_lock (&keycache_lock);
   if (d->m_pgp_key_map.find (recp) == d->m_pgp_key_map.end ())
     {
       // It's enough to look at the PGP Key map. We marked
@@ -828,23 +1140,25 @@ KeyCache::startLocate (const char *addr, Mail *mail) const
                                     NULL);
       CloseHandle (thread);
     }
-  gpgrt_lock_unlock (&keycache_lock);
+  gpgol_unlock (&keycache_lock);
+  TRETURN;
 }
 
 void
 KeyCache::startLocateSecret (const char *addr, Mail *mail) const
 {
+  TSTART;
   if (!addr)
     {
       TRACEPOINT;
-      return;
+      TRETURN;
     }
   std::string recp = GpgME::UserID::addrSpecFromString (addr);
   if (recp.empty ())
     {
-      return;
+      TRETURN;
     }
-  gpgrt_lock_lock (&keycache_lock);
+  gpgol_lock (&keycache_lock);
   if (d->m_pgp_skey_map.find (recp) == d->m_pgp_skey_map.end ())
     {
       // It's enough to look at the PGP Key map. We marked
@@ -859,7 +1173,8 @@ KeyCache::startLocateSecret (const char *addr, Mail *mail) const
                                     NULL);
       CloseHandle (thread);
     }
-  gpgrt_lock_unlock (&keycache_lock);
+  gpgol_unlock (&keycache_lock);
+  TRETURN;
 }
 
 
@@ -890,6 +1205,7 @@ KeyCache::setPgpKeySecret(const std::string &mbox, const GpgME::Key &key)
 bool
 KeyCache::isMailResolvable(Mail *mail)
 {
+  TSTART;
   /* Get the data from the mail. */
   const auto sender = mail->getSender ();
   auto recps = mail->getCachedRecipients ();
@@ -898,7 +1214,7 @@ KeyCache::isMailResolvable(Mail *mail)
     {
       log_debug ("%s:%s: Mail has no sender or no recipients.",
                  SRCNAME, __func__);
-      return false;
+      TRETURN false;
     }
 
 
@@ -907,12 +1223,12 @@ KeyCache::isMailResolvable(Mail *mail)
 
   if (!encKeys.empty())
     {
-      return true;
+      TRETURN true;
     }
 
   if (!opt.enable_smime)
     {
-      return false;
+      TRETURN false;
     }
 
   /* Check S/MIME instead here we need to include the sender
@@ -921,7 +1237,7 @@ KeyCache::isMailResolvable(Mail *mail)
   GpgME::Key sigKey= getSigningKey (sender.c_str(), GpgME::CMS);
   encKeys = getEncryptionKeys (recps, GpgME::CMS);
 
-  return !encKeys.empty() && !sigKey.isNull();
+  TRETURN !encKeys.empty() && !sigKey.isNull();
 }
 
 void
@@ -941,3 +1257,23 @@ KeyCache::onUpdateJobDone (const char *fpr, const GpgME::Key &key)
 {
   return d->onUpdateJobDone (fpr, key);
 }
+
+void
+KeyCache::importFromAddrBook (const std::string &mbox, const char *key_data,
+                              Mail *mail) const
+{
+  return d->importFromAddrBook (mbox, key_data, mail);
+}
+
+void
+KeyCache::onAddrBookImportJobDone (const std::string &mbox,
+                                   const std::vector<std::string> &result_fprs)
+{
+  return d->onAddrBookImportJobDone (mbox, result_fprs);
+}
+
+std::vector<GpgME::Key>
+KeyCache::getOverrides (const std::string &mbox)
+{
+  return d->getPGPOverrides (mbox.c_str ());
+}
