@@ -54,7 +54,8 @@ public:
     /* Get the keys for recipents. The keys
        are taken from the internal cache. If
        one recipient can't be resolved an empty
-       list is returned. */
+       list is returned.
+       */
     std::vector<GpgME::Key> getEncryptionKeys (const std::vector<std::string> &recipients,
                                                GpgME::Protocol proto) const;
 
@@ -63,14 +64,22 @@ public:
 
        The mail argument is used to add / remove the
        locator thread counter.
+
+       async
        */
     void startLocate (const std::vector<std::string> &addrs, Mail *mail) const;
 
-    /* Look for a secret key for the addr. */
+    /* Look for a secret key for the addr.
+
+       async
+    */
     void startLocateSecret (const char *addr, Mail *mail) const;
 
     /* Start a key location in a background thread filling
-       the key cache. */
+       the key cache.
+
+       async
+       */
     void startLocate (const char *addr, Mail *mail) const;
 
     /* Check that a mail is resolvable through the keycache.
@@ -83,6 +92,8 @@ public:
     /* Search / Update a key in the cache. This is meant to be
        called e.g. after a verify to update the key.
 
+       async
+
        A known issue is that a get right after it might
        still return an outdated key but the get after that
        would return the updated one. This is acceptable as
@@ -97,12 +108,31 @@ public:
        to insert keys into the cache */
     GpgME::Key getByFpr (const char *fpr, bool block = true) const;
 
+    /* Import key data from the address book for the address mbox.
+       Keys imported this way take precedence over other keys for
+       this mail address regardless of validity.
+
+       The mail argument is used to add / remove the
+       locator thread counter.
+
+       async
+    */
+    void importFromAddrBook (const std::string &mbox,
+                             const char *key_data,
+                             Mail *mail) const;
+
+    /* Get optional overrides for an address. */
+    std::vector<GpgME::Key> getOverrides (const std::string &mbox);
+
+
     // Internal for thread
     void setSmimeKey(const std::string &mbox, const GpgME::Key &key);
     void setPgpKey(const std::string &mbox, const GpgME::Key &key);
     void setSmimeKeySecret(const std::string &mbox, const GpgME::Key &key);
     void setPgpKeySecret(const std::string &mbox, const GpgME::Key &key);
-    void onUpdateJobDone (const char *fpr, const GpgME::Key &key);
+    void onUpdateJobDone(const char *fpr, const GpgME::Key &key);
+    void onAddrBookImportJobDone (const std::string &fpr,
+                                  const std::vector<std::string> &result_fprs);
 
 private:
 
