@@ -394,7 +394,9 @@ public:
   /** Get the recipients. */
   std::vector<std::string> getRecipients_o () const;
 
-  /** Try to locate the keys for all recipients */
+  /** Try to locate the keys for all recipients.
+      This also triggers the Addressbook integration, which we
+      treat as locate jobs. */
   void locateKeys_o ();
 
   /** State variable to check if a close was triggerd by us. */
@@ -576,7 +578,7 @@ public:
   /* Check if the keys can be resolved automatically and trigger
    * setting the crypto flags accordingly.
    */
-  void autoresolveCheck ();
+  void autosecureCheck ();
 
   /* Set if a mail should be secured (encrypted and signed)
    *
@@ -596,6 +598,9 @@ public:
   /* Gets an additional reference for GetInspector.CurrentItem */
   void refCurrentItem ();
 
+  /* Get the storeID for this mail */
+  std::string storeID() { return m_store_id; }
+
 private:
   void updateCategories_o ();
   void updateSigstate ();
@@ -646,5 +651,8 @@ private:
   bool m_first_autosecure_check; /* This is the first autoresolve check */
   int m_locate_count; /* The number of key locates pending for this mail. */
   bool m_is_about_to_be_moved;
+  bool m_locate_in_progress; /* Simplified state variable for locate */
+  std::string m_store_id; /* Store id for categories */
+  std::string m_verify_category; /* The category string for the verify result */
 };
 #endif // MAIL_H
