@@ -126,6 +126,7 @@ do_delayed_locate (LPVOID arg)
 EVENT_SINK_INVOKE(MailItemEvents)
 {
   USE_INVOKE_ARGS
+  TSTART;
   if (!m_mail)
     {
       m_mail = Mail::getMailForItem (m_object);
@@ -133,7 +134,7 @@ EVENT_SINK_INVOKE(MailItemEvents)
         {
           log_error ("%s:%s: mail event without mail object known. Bug.",
                      SRCNAME, __func__);
-          return S_OK;
+          TRETURN S_OK;
         }
     }
 
@@ -142,19 +143,19 @@ EVENT_SINK_INVOKE(MailItemEvents)
     {
       case Open:
         {
-          log_oom_extra ("%s:%s: Open : %p",
+          log_oom ("%s:%s: Open : %p",
                          SRCNAME, __func__, m_mail);
           int draft_flags = 0;
           if (!opt.encrypt_default && !opt.sign_default)
             {
-              return S_OK;
+              TRETURN S_OK;
             }
           LPMESSAGE message = get_oom_base_message (m_object);
           if (!message)
             {
               log_error ("%s:%s: Failed to get message.",
                          SRCNAME, __func__);
-              break;
+              TBREAK;
             }
           if (opt.encrypt_default)
             {
@@ -166,22 +167,29 @@ EVENT_SINK_INVOKE(MailItemEvents)
             }
           set_gpgol_draft_info_flags (message, draft_flags);
           gpgol_release (message);
-          break;
+          TBREAK;
         }
       case BeforeRead:
         {
-          log_oom_extra ("%s:%s: BeforeRead : %p",
+          log_oom ("%s:%s: BeforeRead : %p",
                          SRCNAME, __func__, m_mail);
+          if (GpgolAddin::get_instance ()->isShutdown())
+            {
+              log_debug ("%s:%s: Ignoring read after shutdown.",
+                         SRCNAME, __func__);
+              TBREAK;
+            }
+
           if (m_mail->preProcessMessage_m ())
             {
               log_error ("%s:%s: Pre process message failed.",
                          SRCNAME, __func__);
             }
-          break;
+          TBREAK;
         }
       case Read:
         {
-          log_oom_extra ("%s:%s: Read : %p",
+          log_oom ("%s:%s: Read : %p",
                          SRCNAME, __func__, m_mail);
           if (!m_mail->isCryptoMail ())
             {
@@ -190,14 +198,14 @@ EVENT_SINK_INVOKE(MailItemEvents)
               /* Ensure that no wrong sigstatus is shown */
               CloseHandle(CreateThread (NULL, 0, delayed_invalidate_ui, (LPVOID) 300, 0,
                                         NULL));
-              break;
+              TBREAK;
             }
           if (m_mail->setUUID_o ())
             {
               log_debug ("%s:%s: Failed to set uuid.",
                          SRCNAME, __func__);
               delete m_mail; /* deletes this, too */
-              return S_OK;
+              TRETURN S_OK;
             }
           if (m_mail->decryptVerify_o ())
             {
@@ -213,7 +221,7 @@ EVENT_SINK_INVOKE(MailItemEvents)
                          SRCNAME, __func__);
               m_mail->setNeedsSave (true);
             }
-          break;
+          TBREAK;
         }
       case PropertyChange:
         {
@@ -223,19 +231,15 @@ EVENT_SINK_INVOKE(MailItemEvents)
             {
               log_error ("%s:%s: Unexpected params.",
                          SRCNAME, __func__);
-              break;
+              TBREAK;
             }
           const wchar_t *prop_name = parms->rgvarg[0].bstrVal;
           if (!m_mail->isCryptoMail ())
             {
-              if (!opt.autoresolve)
-                {
-                  break;
-                }
               if (m_mail->hasOverrideMimeData())
                 {
                   /* This is a mail created by us. Ignore propchanges. */
-                  break;
+                  TBREAK;
                 }
               if (!wcscmp (prop_name, L"To") /* ||
                   !wcscmp (prop_name, L"BCC") ||
@@ -255,7 +259,7 @@ EVENT_SINK_INVOKE(MailItemEvents)
                       CloseHandle(thread);
                     }
                 }
-              break;
+              TBREAK;
             }
           for (const wchar_t **cur = prop_blacklist; *cur; cur++)
             {
@@ -263,7 +267,7 @@ EVENT_SINK_INVOKE(MailItemEvents)
                 {
                   log_oom ("%s:%s: Message %p propchange: %ls discarded.",
                            SRCNAME, __func__, m_object, prop_name);
-                  return S_OK;
+                  TRETURN S_OK;
                 }
             }
           log_oom ("%s:%s: Message %p propchange: %ls.",
@@ -276,12 +280,12 @@ EVENT_SINK_INVOKE(MailItemEvents)
                 {
                   log_debug ("%s:%s: Ignoring SendUsingAccount change for sent %p ",
                              SRCNAME, __func__, m_object);
-                  return S_OK;
+                  TRETURN S_OK;
                 }
               log_debug ("%s:%s: Message %p looks like send again.",
                         SRCNAME, __func__, m_object);
               m_mail->setIsSendAgain (true);
-              return S_OK;
+              TRETURN S_OK;
             }
 
           /* We have tried several scenarios to handle propery changes.
@@ -304,7 +308,7 @@ EVENT_SINK_INVOKE(MailItemEvents)
              the mail was not read by us. */
           if (propchangeWarnShown)
             {
-              return S_OK;
+              TRETURN S_OK;
             }
 
           wchar_t *title = utf8_to_wchar (_("Sorry, that's not possible, yet"));
@@ -323,14 +327,14 @@ EVENT_SINK_INVOKE(MailItemEvents)
           xfree (msg);
           xfree (title);
           propchangeWarnShown = true;
-          return S_OK;
+          TRETURN S_OK;
         }
       case CustomPropertyChange:
         {
-          log_oom_extra ("%s:%s: CustomPropertyChange : %p",
+          log_oom ("%s:%s: CustomPropertyChange : %p",
                          SRCNAME, __func__, m_mail);
           /* TODO */
-          break;
+          TBREAK;
         }
       case Send:
         {
@@ -349,20 +353,20 @@ EVENT_SINK_INVOKE(MailItemEvents)
              If this encryption is successful and we pass the send
              as then the encrypted data is sent.
            */
-          log_oom_extra ("%s:%s: Send : %p",
+          log_oom ("%s:%s: Send : %p",
                          SRCNAME, __func__, m_mail);
           if (!m_mail->needs_crypto_m () && m_mail->cryptState () == Mail::NoCryptMail)
             {
              log_debug ("%s:%s: No crypto neccessary. Passing send for %p obj %p",
                         SRCNAME, __func__, m_mail, m_object);
-             break;
+             TBREAK;
             }
 
           if (parms->cArgs != 1 || parms->rgvarg[0].vt != (VT_BOOL | VT_BYREF))
            {
              log_debug ("%s:%s: Uncancellable send event.",
                         SRCNAME, __func__);
-             break;
+             TBREAK;
            }
 
           if (m_mail->cryptState () == Mail::NoCryptMail &&
@@ -398,7 +402,7 @@ EVENT_SINK_INVOKE(MailItemEvents)
                   // the encryption. We cancel send for our asyncness.
                   // Cancel send
                   *(parms->rgvarg[0].pboolVal) = VARIANT_TRUE;
-                  break;
+                  TBREAK;
                 }
               else
                 {
@@ -409,7 +413,7 @@ EVENT_SINK_INVOKE(MailItemEvents)
                                  "Crypto failed or canceled.",
                                  SRCNAME, __func__, m_object, m_mail);
                       *(parms->rgvarg[0].pboolVal) = VARIANT_TRUE;
-                      break;
+                      TBREAK;
                     }
                   // For inline response we can't trigger send programatically
                   // so we do the encryption in sync.
@@ -429,7 +433,7 @@ EVENT_SINK_INVOKE(MailItemEvents)
                       gpgol_bug (m_mail->getWindow (),
                                  ERR_INLINE_BODY_INV_STATE);
                       *(parms->rgvarg[0].pboolVal) = VARIANT_TRUE;
-                      break;
+                      TBREAK;
                     }
                 }
             }
@@ -445,12 +449,12 @@ EVENT_SINK_INVOKE(MailItemEvents)
                              ERR_WANTS_SEND_INLINE_BODY);
                   m_mail->setCryptState (Mail::NoCryptMail);
                   *(parms->rgvarg[0].pboolVal) = VARIANT_TRUE;
-                  break;
+                  TBREAK;
                 }
               log_debug ("%s:%s: Passing send event for no-mime message %p.",
                          SRCNAME, __func__, m_object);
               WKSHelper::instance()->allow_notify (1000);
-              break;
+              TBREAK;
             }
 
           if (m_mail->cryptState () == Mail::WantsSendMIME)
@@ -477,7 +481,7 @@ EVENT_SINK_INVOKE(MailItemEvents)
                              SRCNAME, __func__, m_object, m_mail);
                   m_mail->setCryptState (Mail::NoCryptMail);
                   *(parms->rgvarg[0].pboolVal) = VARIANT_TRUE;
-                  break;
+                  TBREAK;
 #else
                   log_debug ("%s:%s: Message %p mail %p - "
                              "not encrypted or not empty body detected - MIME.",
@@ -508,7 +512,7 @@ EVENT_SINK_INVOKE(MailItemEvents)
                   log_error ("%s:%s: HrGetOneProp() failed: hr=%#lx\n",
                              SRCNAME, __func__, hr);
                   gpgol_release (message);
-                  break;
+                  TBREAK;
                 }
               if (propval->Value.lpszA && !strstr (propval->Value.lpszA, "GpgOL"))
                 {
@@ -518,27 +522,33 @@ EVENT_SINK_INVOKE(MailItemEvents)
                   log_debug ("%s:%s: Message %p - Activating T3656 Workaround",
                              SRCNAME, __func__, m_object);
                   message = get_oom_base_message (m_object);
-                  // It's important we use the _base_ message here.
-                  mapi_save_changes (message, KEEP_OPEN_READWRITE | FORCE_SAVE);
-                  message->SubmitMessage(0);
-                  gpgol_release (message);
-
+                  if (message)
+                    {
+                      // It's important we use the _base_ message here.
+                      mapi_save_changes (message, FORCE_SAVE);
+                      message->SubmitMessage(0);
+                      gpgol_release (message);
+                      // Close the composer and trigger unloads
+                      CloseHandle(CreateThread (NULL, 0, close_mail, (LPVOID) m_mail, 0,
+                                                NULL));
+                    }
+                  else
+                    {
+                      gpgol_bug (nullptr,
+                                 ERR_GET_BASE_MSG_FAILED);
+                    }
                   // Cancel send
                   *(parms->rgvarg[0].pboolVal) = VARIANT_TRUE;
-
-                  // Close the composer and trigger unloads
-                  CloseHandle(CreateThread (NULL, 0, close_mail, (LPVOID) m_mail, 0,
-                                            NULL));
                 }
               MAPIFreeBuffer (propval);
               if (*(parms->rgvarg[0].pboolVal) == VARIANT_TRUE)
                 {
-                  break;
+                  TBREAK;
                 }
               log_debug ("%s:%s: Passing send event for mime-encrypted message %p.",
                          SRCNAME, __func__, m_object);
               WKSHelper::instance()->allow_notify (1000);
-              break;
+              TBREAK;
             }
           else
             {
@@ -547,11 +557,11 @@ EVENT_SINK_INVOKE(MailItemEvents)
                          SRCNAME, __func__, m_object);
               *(parms->rgvarg[0].pboolVal) = VARIANT_TRUE;
             }
-          return S_OK;
+          TRETURN S_OK;
         }
       case Write:
         {
-          log_oom_extra ("%s:%s: Write : %p",
+          log_oom ("%s:%s: Write : %p",
                          SRCNAME, __func__, m_mail);
           /* This is a bit strange. We sometimes get multiple write events
              without a read in between. When we access the message in
@@ -563,14 +573,14 @@ EVENT_SINK_INVOKE(MailItemEvents)
              /* This happens in the weird case */
              log_debug ("%s:%s: Uncancellable write event.",
                         SRCNAME, __func__);
-             break;
+             TBREAK;
            }
 
           if (m_mail->isAboutToBeMoved())
             {
               log_debug ("%s:%s: Mail is about to be moved. Passing write for %p",
                          SRCNAME, __func__, m_mail);
-              break;
+              TBREAK;
             }
 
           if (m_mail->isCryptoMail () && !m_mail->needsSave ())
@@ -615,7 +625,7 @@ EVENT_SINK_INVOKE(MailItemEvents)
                       // Mail::clearLastMail ();
 
                       do_in_ui_thread_async (REVERT_MAIL, m_mail);
-                      return S_OK;
+                      TRETURN S_OK;
                     }
                 }
               /* We cancel the write event to stop outlook from excessively
@@ -627,7 +637,7 @@ EVENT_SINK_INVOKE(MailItemEvents)
               *(parms->rgvarg[0].pboolVal) = VARIANT_TRUE;
               log_debug ("%s:%s: Canceling write event.",
                          SRCNAME, __func__);
-              return S_OK;
+              TRETURN S_OK;
             }
 
           if (m_mail->isCryptoMail () && m_mail->needsSave () &&
@@ -665,11 +675,11 @@ EVENT_SINK_INVOKE(MailItemEvents)
           log_debug ("%s:%s: Passing write event.",
                      SRCNAME, __func__);
           m_mail->setNeedsSave (false);
-          break;
+          TBREAK;
         }
       case AfterWrite:
         {
-          log_oom_extra ("%s:%s: AfterWrite : %p",
+          log_oom ("%s:%s: AfterWrite : %p",
                          SRCNAME, __func__, m_mail);
           if (m_mail->cryptState () == Mail::NeedsFirstAfterWrite)
             {
@@ -677,23 +687,23 @@ EVENT_SINK_INVOKE(MailItemEvents)
               m_mail->setCryptState (Mail::NeedsActualCrypt);
               if (m_mail->encryptSignStart_o ())
                 {
-                  log_debug ("%s:%s: Encrypt sign start failes.",
+                  log_debug ("%s:%s: Encrypt sign start failed.",
                              SRCNAME, __func__);
                   m_mail->setCryptState (Mail::NoCryptMail);
                 }
-              return S_OK;
+              TRETURN S_OK;
             }
           if (m_mail->cryptState () == Mail::NeedsSecondAfterWrite)
             {
               m_mail->setCryptState (Mail::NeedsUpdateInMAPI);
               m_mail->updateCryptMAPI_m ();
-              return S_OK;
+              TRETURN S_OK;
             }
-          break;
+          TBREAK;
         }
       case Close:
         {
-          log_oom_extra ("%s:%s: Close : %p",
+          log_oom ("%s:%s: Close : %p",
                          SRCNAME, __func__, m_mail);
           if (m_mail->isCryptoMail ())
             {
@@ -710,16 +720,16 @@ EVENT_SINK_INVOKE(MailItemEvents)
                   /* This happens in the weird case */
                   log_debug ("%s:%s: Uncancellable close event.",
                              SRCNAME, __func__);
-                  break;
+                  TBREAK;
                 }
               if (m_mail->getCloseTriggered ())
                 {
                   /* Our close with discard changes, pass through */
                   m_mail->setCloseTriggered (false);
-                  return S_OK;
+                  TRETURN S_OK;
                 }
               *(parms->rgvarg[0].pboolVal) = VARIANT_TRUE;
-              log_oom_extra ("%s:%s: Canceling close event.",
+              log_oom ("%s:%s: Canceling close event.",
                              SRCNAME, __func__);
               if (Mail::close(m_mail))
                 {
@@ -727,19 +737,19 @@ EVENT_SINK_INVOKE(MailItemEvents)
                              SRCNAME, __func__);
                 }
             }
-          return S_OK;
+          TRETURN S_OK;
         }
       case Unload:
         {
-          log_oom_extra ("%s:%s: Unload : %p",
+          log_oom ("%s:%s: Unload : %p",
                          SRCNAME, __func__, m_mail);
           log_debug ("%s:%s: Removing Mail for message: %p.",
                      SRCNAME, __func__, m_object);
           delete m_mail;
-          log_oom_extra ("%s:%s: deletion done",
+          log_oom ("%s:%s: deletion done",
                          SRCNAME, __func__);
           memdbg_dump ();
-          return S_OK;
+          TRETURN S_OK;
         }
       /* Fallthrough */
       case ReplyAll:
@@ -749,7 +759,7 @@ EVENT_SINK_INVOKE(MailItemEvents)
         }
       case Forward:
         {
-          log_oom_extra ("%s:%s: %s : %p",
+          log_oom ("%s:%s: %s : %p",
                          SRCNAME, __func__, is_reply ? "reply" : "forward", m_mail);
           int draft_flags = 0;
           if (opt.encrypt_default)
@@ -794,7 +804,7 @@ EVENT_SINK_INVOKE(MailItemEvents)
           if (!is_crypto_mail)
             {
               /* Replys to non crypto mails do not interest us anymore. */
-              break;
+              TBREAK;
             }
 
           Mail *last_mail = Mail::getLastMail ();
@@ -865,16 +875,16 @@ EVENT_SINK_INVOKE(MailItemEvents)
               Mail::clearLastMail ();
             }
 
-          log_oom_extra ("%s:%s: Reply Forward ReplyAll: %p",
+          log_oom ("%s:%s: Reply Forward ReplyAll: %p",
                          SRCNAME, __func__, m_mail);
           if (!opt.reply_crypt)
             {
-              break;
+              TBREAK;
             }
           int crypto_flags = 0;
           if (!(crypto_flags = m_mail->getCryptoFlags ()))
             {
-              break;
+              TBREAK;
             }
           if (parms->cArgs != 2 || parms->rgvarg[1].vt != (VT_DISPATCH) ||
               parms->rgvarg[0].vt != (VT_BOOL | VT_BYREF))
@@ -883,27 +893,27 @@ EVENT_SINK_INVOKE(MailItemEvents)
               log_debug ("%s:%s: Unexpected args %i %x %x named: %i",
                          SRCNAME, __func__, parms->cArgs, parms->rgvarg[0].vt, parms->rgvarg[1].vt,
                          parms->cNamedArgs);
-              break;
+              TBREAK;
             }
           LPMESSAGE msg = get_oom_base_message (parms->rgvarg[1].pdispVal);
           if (!msg)
             {
               log_debug ("%s:%s: Failed to get base message",
                          SRCNAME, __func__);
-              break;
+              TBREAK;
             }
           set_gpgol_draft_info_flags (msg, crypto_flags);
           gpgol_release (msg);
-          break;
+          TBREAK;
         }
       case AttachmentRemove:
         {
-          log_oom_extra ("%s:%s: AttachmentRemove: %p",
+          log_oom ("%s:%s: AttachmentRemove: %p",
                          SRCNAME, __func__, m_mail);
           if (!m_mail->isCryptoMail () || attachRemoveWarnShown ||
               m_mail->attachmentRemoveWarningDisabled ())
             {
-              return S_OK;
+              TRETURN S_OK;
             }
           gpgol_message_box (get_active_hwnd (),
                              _("Attachments are part of the crypto message.\nThey "
@@ -911,13 +921,13 @@ EVENT_SINK_INVOKE(MailItemEvents)
                                "time this message is opened."),
                              _("Sorry, that's not possible, yet"), MB_OK);
           attachRemoveWarnShown = true;
-          return S_OK;
+          TRETURN S_OK;
         }
 
       default:
-        log_oom_extra ("%s:%s: Message:%p Unhandled Event: %lx \n",
+        log_oom ("%s:%s: Message:%p Unhandled Event: %lx \n",
                        SRCNAME, __func__, m_object, dispid);
     }
-  return S_OK;
+  TRETURN S_OK;
 }
 END_EVENT_SINK(MailItemEvents, IID_MailItemEvents)
