@@ -195,6 +195,57 @@ get_conf_bool (const char *name, int defaultVal)
   return ret;
 }
 
+static int
+dbg_compat (int oldval)
+{
+  // We broke the debug levels at some point
+  // This is cmpatibility code with the old
+  // levels.
+
+#define DBG_MEMORY_OLD     (1<<5) // 32
+#define DBG_MIME_PARSER_OLD (1<<7) // 128 Unified as DBG_DATA
+#define DBG_MIME_DATA_OLD   (1<<8) // 256 Unified in read_options
+#define DBG_OOM_OLD        (1<<9) // 512 Unified as DBG_OOM
+#define DBG_OOM_EXTRA_OLD  (1<<10)// 1024 Unified in read_options
+  int new_dbg = oldval;
+  if ((oldval & DBG_MEMORY_OLD))
+    {
+      new_dbg |= DBG_MEMORY;
+      new_dbg -= DBG_MEMORY_OLD;
+    }
+  if ((oldval & DBG_OOM_OLD))
+    {
+      new_dbg |= DBG_OOM;
+      new_dbg -= DBG_OOM_OLD;
+    }
+  if ((oldval & DBG_MIME_PARSER_OLD))
+    {
+      new_dbg |= DBG_DATA;
+      new_dbg -= DBG_MIME_PARSER_OLD;
+    }
+  if ((oldval & DBG_MIME_DATA_OLD))
+    {
+      new_dbg |= DBG_DATA;
+      new_dbg -= DBG_MIME_DATA_OLD;
+    }
+  if ((oldval & DBG_OOM_OLD))
+    {
+      new_dbg |= DBG_OOM;
+      new_dbg -= DBG_OOM_OLD;
+    }
+  if ((oldval & DBG_OOM_EXTRA_OLD))
+    {
+      new_dbg |= DBG_OOM;
+      new_dbg -= DBG_OOM_EXTRA_OLD;
+    }
+#undef DBG_MEMORY_OLD
+#undef DBG_MIME_PARSER_OLD
+#undef DBG_MIME_DATA_OLD
+#undef DBG_OOM_OLD
+#undef DBG_OOM_EXTRA_OLD
+  return new_dbg;
+}
+
 /* Read option settings from the Registry. */
 void
 read_options (void)
@@ -224,27 +275,20 @@ read_options (void)
               pend += strspn (pend, ", \t\n\r\f");
             }
           if (isascii (*p) && isdigit (*p))
-            opt.enable_debug |= strtoul (p, NULL, 0);
-          else if (!strcmp (p, "ioworker"))
-            opt.enable_debug |= DBG_IOWORKER;
-          else if (!strcmp (p, "ioworker-extra"))
-            opt.enable_debug |= DBG_IOWORKER_EXTRA;
-          else if (!strcmp (p, "filter"))
-            opt.enable_debug |= DBG_FILTER;
-          else if (!strcmp (p, "filter-extra"))
-            opt.enable_debug |= DBG_FILTER_EXTRA;
+            {
+              opt.enable_debug |= dbg_compat (strtoul (p, NULL, 0));
+
+            }
           else if (!strcmp (p, "memory"))
             opt.enable_debug |= DBG_MEMORY;
-          else if (!strcmp (p, "commands"))
-            opt.enable_debug |= DBG_COMMANDS;
           else if (!strcmp (p, "mime-parser"))
-            opt.enable_debug |= DBG_MIME_PARSER;
+            opt.enable_debug |= DBG_DATA;
           else if (!strcmp (p, "mime-data"))
-            opt.enable_debug |= DBG_MIME_DATA;
+            opt.enable_debug |= DBG_DATA;
           else if (!strcmp (p, "oom"))
             opt.enable_debug |= DBG_OOM;
           else if (!strcmp (p, "oom-extra"))
-            opt.enable_debug |= DBG_OOM_EXTRA;
+            opt.enable_debug |= DBG_OOM;
           else
             log_debug ("invalid debug flag `%s' ignored", p);
         }
@@ -260,17 +304,10 @@ read_options (void)
      as the option for debuging was not read before. */
   free (val); val = NULL;
   if (opt.enable_debug)
-    log_debug ("enabled debug flags:%s%s%s%s%s%s%s%s%s%s\n",
-               (opt.enable_debug & DBG_IOWORKER)? " ioworker":"",
-               (opt.enable_debug & DBG_IOWORKER_EXTRA)? " ioworker-extra":"",
-               (opt.enable_debug & DBG_FILTER)? " filter":"",
-               (opt.enable_debug & DBG_FILTER_EXTRA)? " filter-extra":"",
+    log_debug ("enabled debug flags:%s%s%s\n",
                (opt.enable_debug & DBG_MEMORY)? " memory":"",
-               (opt.enable_debug & DBG_COMMANDS)? " commands":"",
-               (opt.enable_debug & DBG_MIME_PARSER)? " mime-parser":"",
-               (opt.enable_debug & DBG_MIME_DATA)? " mime-data":"",
-               (opt.enable_debug & DBG_OOM)? " oom":"",
-               (opt.enable_debug & DBG_OOM_EXTRA)? " oom-extra":""
+               (opt.enable_debug & DBG_DATA)? " data":"",
+               (opt.enable_debug & DBG_OOM)? " oom":""
                );
 
   opt.enable_smime = get_conf_bool ("enableSmime", 0);
@@ -280,10 +317,11 @@ read_options (void)
   opt.reply_crypt = get_conf_bool ("replyCrypt", 1);
   opt.prefer_smime = get_conf_bool ("preferSmime", 0);
   opt.autoresolve = get_conf_bool ("autoresolve", 1);
+  opt.autoretrieve = get_conf_bool ("autoretrieve", 0);
   opt.automation = get_conf_bool ("automation", 1);
   opt.autosecure = get_conf_bool ("autosecure", 1);
   opt.autotrust = get_conf_bool ("autotrust", 0);
-  opt.sync_enc = get_conf_bool ("syncEnc", 0);
+  opt.search_smime_servers = get_conf_bool ("searchSmimeServers", 0);
   opt.smime_html_warn_shown = get_conf_bool ("smimeHtmlWarnShown", 0);
 
   if (!opt.automation)
@@ -293,7 +331,12 @@ read_options (void)
       opt.autosecure = 0;
       opt.autoresolve = 0;
       opt.autotrust = 0;
+      opt.autoretrieve = 0;
     }
+
+  /* Hidden options  */
+  opt.sync_enc = get_conf_bool ("syncEnc", 0);
+  opt.sync_dec = get_conf_bool ("syncDec", 0);
 }
 
 
