@@ -129,33 +129,40 @@ debug_message_event (rfc822parse_event_t event)
     case RFC822PARSE_EPILOGUE: s= "Epilogue"; break;
     default: s= "[unknown event]"; break;
     }
-  log_mime_parser ("%s: rfc822 event %s\n", SRCNAME, s);
+  log_data ("%s: rfc822 event %s\n", SRCNAME, s);
 }
 
-/* Returns true if the BER encoded data in BUFFER is CMS signed data.
+/* returns true if the BER encoded data in BUFFER is CMS signed data.
    LENGTH gives the length of the buffer, for correct detection LENGTH
    should be at least about 24 bytes.  */
 #if 0
 static int
 is_cms_signed_data (const char *buffer, size_t length)
 {
+  TSTART;
   const char *p = buffer;
   size_t n = length;
   tlvinfo_t ti;
 
   if (parse_tlv (&p, &n, &ti))
-    return 0;
+    {
+      TRETURN 0;
+    }
   if (!(ti.cls == ASN1_CLASS_UNIVERSAL && ti.tag == ASN1_TAG_SEQUENCE
         && ti.is_cons) )
-    return 0;
+    TRETURN 0;
   if (parse_tlv (&p, &n, &ti))
-    return 0;
+    {
+      TRETURN 0;
+    }
   if (!(ti.cls == ASN1_CLASS_UNIVERSAL && ti.tag == ASN1_TAG_OBJECT_ID
         && !ti.is_cons && ti.length) || ti.length > n)
-    return 0;
+    TRETURN 0;
   if (ti.length == 9 && !memcmp (p, "\x2A\x86\x48\x86\xF7\x0D\x01\x07\x02", 9))
-    return 1;
-  return 0;
+    {
+      TRETURN 1;
+    }
+  TRETURN 0;
 }
 #endif
 
@@ -170,6 +177,7 @@ is_cms_signed_data (const char *buffer, size_t length)
 static int
 t2body (MimeDataProvider *provider, rfc822parse_t msg)
 {
+  TSTART;
   rfc822parse_field_t field;
   mime_context_t ctx = provider->mime_context ();
   const char *ctmain, *ctsub;
@@ -181,6 +189,7 @@ t2body (MimeDataProvider *provider, rfc822parse_t msg)
   char *filename = NULL;
   char *cid = NULL;
   char *charset = NULL;
+  bool ignore_cid = false;
 
   /* Figure out the encoding.  */
   ctx->is_qp_encoded = 0;
@@ -207,6 +216,14 @@ t2body (MimeDataProvider *provider, rfc822parse_t msg)
         filename = rfc2047_parse (s);
       s = rfc822parse_query_parameter (field, NULL, 1);
 
+      if (s && strstr (s, "attachment"))
+        {
+          log_debug ("%s:%s: Found Content-Disposition attachment."
+                     " Ignoring content-id to avoid hiding.",
+                     SRCNAME, __func__);
+          ignore_cid = true;
+        }
+
       /* This is a bit of a taste matter how to treat inline
          attachments. Outlook does not show them inline so we
          should not put it in the body either as we have
@@ -244,7 +261,7 @@ t2body (MimeDataProvider *provider, rfc822parse_t msg)
       ctsub  = "plain";
     }
 
-  log_mime_parser ("%s:%s: ctx=%p, ct=`%s/%s'\n",
+  log_data ("%s:%s: ctx=%p, ct=`%s/%s'\n",
                    SRCNAME, __func__, ctx, ctmain, ctsub);
 
   s = rfc822parse_query_parameter (field, "charset", 0);
@@ -261,13 +278,15 @@ t2body (MimeDataProvider *provider, rfc822parse_t msg)
     }
 
   /* Parse a Content Id header */
-  p = rfc822parse_get_field (msg, "Content-Id", -1, &off);
-  if (p)
+  if (!ignore_cid)
     {
-       cid = xstrdup (p+off);
-       xfree (p);
+      p = rfc822parse_get_field (msg, "Content-Id", -1, &off);
+      if (p)
+        {
+           cid = xstrdup (p+off);
+           xfree (p);
+        }
     }
-
   /* Update our idea of the entire MIME structure.  */
   {
     mimestruct_item_t ms;
@@ -337,7 +356,7 @@ t2body (MimeDataProvider *provider, rfc822parse_t msg)
          sig_data has not been set yet).  We also do this only while
          in verify mode because we don't want to write a full MUA.  */
       ctx->collect_signature = 1;
-      log_mime_parser ("%s:%s: Collecting signature.",
+      log_data ("%s:%s: Collecting signature.",
                        SRCNAME, __func__);
     }
   else if (ctx->nesting_level == 1 && ctx->is_encrypted
@@ -345,7 +364,7 @@ t2body (MimeDataProvider *provider, rfc822parse_t msg)
            && (ctx->protocol == PROTOCOL_OPENPGP
                && !strcmp (ctsub, "octet-stream")))
     {
-      log_mime_parser ("%s:%s: Collecting encrypted PGP data.",
+      log_data ("%s:%s: Collecting encrypted PGP data.",
                        SRCNAME, __func__);
       ctx->collect_crypto_data = 1;
     }
@@ -358,7 +377,7 @@ t2body (MimeDataProvider *provider, rfc822parse_t msg)
           && (!strcmp (ctsub, "pkcs7-mime")
               || !strcmp (ctsub, "x-pkcs7-mime")))
         {
-          log_mime_parser ("%s:%s: Collecting crypted S/MIME data.",
+          log_data ("%s:%s: Collecting crypted S/MIME data.",
                            SRCNAME, __func__);
           ctx->collect_crypto_data = 1;
         }
@@ -366,7 +385,7 @@ t2body (MimeDataProvider *provider, rfc822parse_t msg)
   rfc822parse_release_field (field); /* (Content-type) */
   ctx->in_data = 1;
 
-  log_mime_parser ("%s:%s: this body: nesting=%d partno=%d is_text=%d"
+  log_data ("%s:%s: this body: nesting=%d partno=%d is_text=%d"
                    " charset=\"%s\"\n body_seen=%d is_text_attachment=%d",
                    SRCNAME, __func__,
                    ctx->nesting_level, ctx->part_counter, is_text,
@@ -406,11 +425,11 @@ t2body (MimeDataProvider *provider, rfc822parse_t msg)
       ctx->current_attachment = provider->create_attachment();
       ctx->collect_body = 0;
       ctx->collect_html_body = 0;
-      log_mime_parser ("%s:%s: Collecting attachment.",
+      log_data ("%s:%s: Collecting attachment.",
                        SRCNAME, __func__);
     }
 
-  return 0;
+  TRETURN 0;
 }
 
 static int
@@ -489,16 +508,19 @@ MimeDataProvider::MimeDataProvider(bool no_headers) :
   m_has_html_body(false),
   m_collect_everything(no_headers)
 {
+  TSTART;
   memdbg_ctor ("MimeDataProvider");
   m_mime_ctx = (mime_context_t) xcalloc (1, sizeof *m_mime_ctx);
   m_mime_ctx->msg = rfc822parse_open (message_cb, this);
   m_mime_ctx->mimestruct_tail = &m_mime_ctx->mimestruct;
+  TRETURN;
 }
 
 #ifdef HAVE_W32_SYSTEM
 MimeDataProvider::MimeDataProvider(LPSTREAM stream, bool no_headers):
   MimeDataProvider(no_headers)
 {
+  TSTART;
   if (stream)
     {
       stream->AddRef ();
@@ -507,25 +529,29 @@ MimeDataProvider::MimeDataProvider(LPSTREAM stream, bool no_headers):
   else
     {
       log_error ("%s:%s called without stream ", SRCNAME, __func__);
-      return;
+      TRETURN;
     }
-  log_mime_parser ("%s:%s Collecting data.", SRCNAME, __func__);
+  log_data ("%s:%s Collecting data.", SRCNAME, __func__);
   collect_data (stream);
-  log_mime_parser ("%s:%s Data collected.", SRCNAME, __func__);
+  log_data ("%s:%s Data collected.", SRCNAME, __func__);
   gpgol_release (stream);
+  TRETURN;
 }
 #endif
 
 MimeDataProvider::MimeDataProvider(FILE *stream, bool no_headers):
   MimeDataProvider(no_headers)
 {
-  log_mime_parser ("%s:%s Collecting data from file.", SRCNAME, __func__);
+  TSTART;
+  log_data ("%s:%s Collecting data from file.", SRCNAME, __func__);
   collect_data (stream);
-  log_mime_parser ("%s:%s Data collected.", SRCNAME, __func__);
+  log_data ("%s:%s Data collected.", SRCNAME, __func__);
+  TRETURN;
 }
 
 MimeDataProvider::~MimeDataProvider()
 {
+  TSTART;
   memdbg_dtor ("MimeDataProvider");
   log_debug ("%s:%s", SRCNAME, __func__);
   while (m_mime_ctx->mimestruct)
@@ -544,6 +570,7 @@ MimeDataProvider::~MimeDataProvider()
     {
       delete m_signature;
     }
+  TRETURN;
 }
 
 bool
@@ -558,34 +585,36 @@ MimeDataProvider::isSupported(GpgME::DataProvider::Operation op) const
 ssize_t
 MimeDataProvider::read(void *buffer, size_t size)
 {
-  log_mime_parser ("%s:%s: Reading: " SIZE_T_FORMAT "Bytes",
+  TSTART;
+  log_data ("%s:%s: Reading: " SIZE_T_FORMAT "Bytes",
                  SRCNAME, __func__, size);
   ssize_t bRead = m_crypto_data.read (buffer, size);
-  if (opt.enable_debug & DBG_MIME_DATA && bRead)
+  if ((opt.enable_debug & DBG_DATA) && bRead)
     {
       std::string buf ((char *)buffer, bRead);
 
       if (!is_binary (buf))
         {
-          log_mime_data ("%s:%s: Data: \n------\n%s\n------",
+          log_data ("%s:%s: Data: \n------\n%s\n------",
                            SRCNAME, __func__, buf.c_str());
         }
       else
         {
-          log_mime_data ("%s:%s: Hex Data: \n------\n%s\n------",
+          log_data ("%s:%s: Hex Data: \n------\n%s\n------",
                            SRCNAME, __func__,
                            string_to_hex (buf).c_str ());
         }
     }
-  return bRead;
+  TRETURN bRead;
 }
 
 /* Split some raw data into lines and handle them accordingly.
-   returns the amount of bytes not taken from the input buffer.
+   Returns the amount of bytes not taken from the input buffer.
 */
 size_t
 MimeDataProvider::collect_input_lines(const char *input, size_t insize)
 {
+  TSTART;
   char linebuf[LINEBUFSIZE];
   const char *s = input;
   size_t pos = 0;
@@ -601,7 +630,7 @@ MimeDataProvider::collect_input_lines(const char *input, size_t insize)
           log_error ("%s:%s: rfc822 parser failed: line too long\n",
                      SRCNAME, __func__);
           GpgME::Error::setSystemError (GPG_ERR_EIO);
-          return not_taken;
+          TRETURN not_taken;
         }
       if (*s != '\n')
         linebuf[pos++] = *s;
@@ -614,7 +643,7 @@ MimeDataProvider::collect_input_lines(const char *input, size_t insize)
               pos--;
             }
 
-          log_mime_data ("%s:%s: Parsing line=`%.*s'\n",
+          log_data ("%s:%s: Parsing line=`%.*s'\n",
                          SRCNAME, __func__, (int)pos, linebuf);
           /* Check the next state */
           if (rfc822parse_insert (m_mime_ctx->msg,
@@ -623,7 +652,7 @@ MimeDataProvider::collect_input_lines(const char *input, size_t insize)
             {
               log_error ("%s:%s: rfc822 parser failed: %s\n",
                          SRCNAME, __func__, strerror (errno));
-              return not_taken;
+              TRETURN not_taken;
             }
 
           /* Check if the first line of the body is actually
@@ -653,7 +682,7 @@ MimeDataProvider::collect_input_lines(const char *input, size_t insize)
                 {
                   m_crypto_data.write ("\r\n", 2);
                 }
-              log_mime_data ("Writing raw crypto data: %.*s",
+              log_data ("Writing raw crypto data: %.*s",
                                (int)pos, linebuf);
               m_crypto_data.write (linebuf, pos);
               m_mime_ctx->collect_crypto_data = 2;
@@ -717,7 +746,7 @@ MimeDataProvider::collect_input_lines(const char *input, size_t insize)
                 }
               else
                 {
-                  log_mime_parser ("%s:%s Collecting ended / failed.",
+                  log_data ("%s:%s Collecting ended / failed.",
                                    SRCNAME, __func__);
                 }
             }
@@ -769,16 +798,17 @@ MimeDataProvider::collect_input_lines(const char *input, size_t insize)
           pos = 0;
         }
     }
-  return not_taken;
+  TRETURN not_taken;
 }
 
 #ifdef HAVE_W32_SYSTEM
 void
 MimeDataProvider::collect_data(LPSTREAM stream)
 {
+  TSTART;
   if (!stream)
     {
-      return;
+      TRETURN;
     }
   HRESULT hr;
   char buf[BUFSIZE];
@@ -791,11 +821,11 @@ MimeDataProvider::collect_data(LPSTREAM stream)
     {
       if (!bRead)
         {
-          log_mime_parser ("%s:%s: Input stream at EOF.",
+          log_data ("%s:%s: Input stream at EOF.",
                            SRCNAME, __func__);
           break;
         }
-      log_mime_parser ("%s:%s: Read %lu bytes.",
+      log_data ("%s:%s: Read %lu bytes.",
                        SRCNAME, __func__, bRead);
       allRead += bRead;
       if (first_read)
@@ -849,7 +879,7 @@ MimeDataProvider::collect_data(LPSTREAM stream)
           /* For S/MIME, Clearsigned, PGP MESSAGES we just pass everything
              on. Only the Multipart classes need parsing. And the output
              of course. */
-          log_mime_parser ("%s:%s: Just copying data.",
+          log_data ("%s:%s: Just copying data.",
                            SRCNAME, __func__);
           m_crypto_data.write ((void*)buf, (size_t) bRead);
           continue;
@@ -865,7 +895,7 @@ MimeDataProvider::collect_data(LPSTREAM stream)
                      SRCNAME, __func__);
           break;
         }
-      log_mime_parser ("%s:%s: Consumed: " SIZE_T_FORMAT " bytes",
+      log_data ("%s:%s: Consumed: " SIZE_T_FORMAT " bytes",
                        SRCNAME, __func__, m_rawbuf.size() - not_taken);
       m_rawbuf.erase (0, m_rawbuf.size() - not_taken);
     }
@@ -902,7 +932,7 @@ MimeDataProvider::collect_data(LPSTREAM stream)
             }
           if (line.find (':') != std::string::npos)
             {
-              log_mime_parser ("%s:%s: Removing comment '%s'.",
+              log_data ("%s:%s: Removing comment '%s'.",
                                SRCNAME, __func__, line.c_str ());
               continue;
             }
@@ -910,21 +940,23 @@ MimeDataProvider::collect_data(LPSTREAM stream)
           m_crypto_data.write (line.c_str (), line.size ());
         }
     }
+  TRETURN;
 }
 #endif
 
 void
 MimeDataProvider::collect_data(FILE *stream)
 {
+  TSTART;
   if (!stream)
     {
-      return;
+      TRETURN;
     }
   char buf[BUFSIZE];
   size_t bRead;
   while ((bRead = fread (buf, 1, BUFSIZE, stream)) > 0)
     {
-      log_mime_parser ("%s:%s: Read " SIZE_T_FORMAT " bytes.",
+      log_data ("%s:%s: Read " SIZE_T_FORMAT " bytes.",
                        SRCNAME, __func__, bRead);
 
       if (m_collect_everything)
@@ -932,7 +964,7 @@ MimeDataProvider::collect_data(FILE *stream)
           /* For S/MIME, Clearsigned, PGP MESSAGES we just pass everything
              on. Only the Multipart classes need parsing. And the output
              of course. */
-          log_mime_parser ("%s:%s: Making verbatim copy" SIZE_T_FORMAT " bytes.",
+          log_data ("%s:%s: Making verbatim copy" SIZE_T_FORMAT " bytes.",
                            SRCNAME, __func__, bRead);
           m_crypto_data.write ((void*)buf, bRead);
           continue;
@@ -946,24 +978,26 @@ MimeDataProvider::collect_data(FILE *stream)
           log_error ("%s:%s: Collect failed to consume anything.\n"
                      "Buffer too small?",
                      SRCNAME, __func__);
-          return;
+          TRETURN;
         }
-      log_mime_parser ("%s:%s: Consumed: " SIZE_T_FORMAT " bytes",
+      log_data ("%s:%s: Consumed: " SIZE_T_FORMAT " bytes",
                        SRCNAME, __func__, m_rawbuf.size() - not_taken);
       m_rawbuf.erase (0, m_rawbuf.size() - not_taken);
     }
+  TRETURN;
 }
 
 ssize_t MimeDataProvider::write(const void *buffer, size_t bufSize)
 {
+  TSTART;
   if (m_collect_everything)
     {
       /* Writing with collect everything one means that we are outputprovider.
          In this case for inline messages we want to collect everything. */
-      log_mime_parser ("%s:%s: Using complete input as body " SIZE_T_FORMAT " bytes.",
+      log_data ("%s:%s: Using complete input as body " SIZE_T_FORMAT " bytes.",
                        SRCNAME, __func__, bufSize);
       m_body += std::string ((const char *) buffer, bufSize);
-      return bufSize;
+      TRETURN bufSize;
     }
   m_rawbuf += std::string ((const char*)buffer, bufSize);
   size_t not_taken = collect_input_lines (m_rawbuf.c_str(),
@@ -974,12 +1008,12 @@ ssize_t MimeDataProvider::write(const void *buffer, size_t bufSize)
       log_error ("%s:%s: Write failed to consume anything.\n"
                  "Buffer too small? or no newlines in text?",
                  SRCNAME, __func__);
-      return bufSize;
+      TRETURN bufSize;
     }
-  log_mime_parser ("%s:%s: Write Consumed: " SIZE_T_FORMAT " bytes",
+  log_data ("%s:%s: Write Consumed: " SIZE_T_FORMAT " bytes",
                    SRCNAME, __func__, m_rawbuf.size() - not_taken);
   m_rawbuf.erase (0, m_rawbuf.size() - not_taken);
-  return bufSize;
+  TRETURN bufSize;
 }
 
 off_t
@@ -991,13 +1025,15 @@ MimeDataProvider::seek(off_t offset, int whence)
 GpgME::Data *
 MimeDataProvider::signature() const
 {
-  return m_signature;
+  TSTART;
+  TRETURN m_signature;
 }
 
 std::shared_ptr<Attachment>
 MimeDataProvider::create_attachment()
 {
-  log_mime_parser ("%s:%s: Creating attachment.",
+  TSTART;
+  log_data ("%s:%s: Creating attachment.",
                    SRCNAME, __func__);
 
   auto attach = std::shared_ptr<Attachment> (new Attachment());
@@ -1015,7 +1051,7 @@ MimeDataProvider::create_attachment()
         }
       else
         {
-          log_mime_parser ("%s:%s: Attachment filename: %s",
+          log_data ("%s:%s: Attachment filename: %s",
                            SRCNAME, __func__, m_mime_ctx->mimestruct_cur->filename);
           attach->set_display_name (m_mime_ctx->mimestruct_cur->filename);
         }
@@ -1026,50 +1062,48 @@ MimeDataProvider::create_attachment()
     }
   m_attachments.push_back (attach);
 
-  return attach;
+  TRETURN attach;
   /* TODO handle encoding */
 }
 
+void MimeDataProvider::finalize ()
+{
+  TSTART;
+  if (m_rawbuf.size ())
+    {
+      m_rawbuf += "\r\n";
+      size_t not_taken = collect_input_lines (m_rawbuf.c_str(),
+                                              m_rawbuf.size());
+      m_rawbuf.erase (0, m_rawbuf.size() - not_taken);
+      if (m_rawbuf.size ())
+        {
+          log_error ("%s:%s: Collect left data in buffer.\n",
+                     SRCNAME, __func__);
+        }
+    }
+  TRETURN;
+}
+
 const std::string &MimeDataProvider::get_body ()
 {
-  if (m_rawbuf.size())
-    {
-      /* If there was some data left in the rawbuf this could
-         mean that some plaintext was not finished with a linefeed.
-         In that case we append it to the bodies. */
-      m_body += m_rawbuf;
-      m_html_body += m_rawbuf;
-      m_rawbuf.clear();
-    }
-  return m_body;
+  TSTART;
+  TRETURN m_body;
 }
 
 const std::string &MimeDataProvider::get_html_body ()
 {
-  if (!m_has_html_body)
-    {
-      /* Don't do the last line handling if we don't
-         have html */
-      return m_html_body;
-    }
-  if (m_rawbuf.size())
-    {
-      /* If there was some data left in the rawbuf this could
-         mean that some plaintext was not finished with a linefeed.
-         In that case we append it to the bodies. */
-      m_body += m_rawbuf;
-      m_html_body += m_rawbuf;
-      m_rawbuf.clear();
-    }
-  return m_html_body;
+  TSTART;
+  TRETURN m_html_body;
 }
 
 const std::string &MimeDataProvider::get_html_charset() const
 {
-  return m_html_charset;
+  TSTART;
+  TRETURN m_html_charset;
 }
 
 const std::string &MimeDataProvider::get_body_charset() const
 {
-  return m_body_charset;
+  TSTART;
+  TRETURN m_body_charset;
 }
