@@ -24,9 +24,11 @@
 
 #include <unknwn.h>
 #include "mymapi.h"
+#include "common.h"
 
 #include <vector>
 #include <string>
+#include <memory>
 
 #define MSOCONTROLBUTTON    1
 #define MSOCONTROLEDIT      2
@@ -127,6 +129,9 @@ DEFINE_OLEGUID(IID_IOleWindow,                0x00000114, 0, 0);
   "http://schemas.microsoft.com/mapi/proptag/0x370E001F"
 #define PR_ATTACH_CONTENT_ID_DASL \
   "http://schemas.microsoft.com/mapi/proptag/0x3712001F"
+#define PR_ATTACH_FLAGS_DASL \
+  "http://schemas.microsoft.com/mapi/proptag/0x37140003"
+
 #define PR_TAG_SENDER_SMTP_ADDRESS \
   "http://schemas.microsoft.com/mapi/proptag/0x5D01001F"
 #define PR_TAG_RECEIVED_REPRESENTING_SMTP_ADDRESS \
@@ -138,8 +143,20 @@ DEFINE_OLEGUID(IID_IOleWindow,                0x00000114, 0, 0);
 #define PR_SENT_REPRESENTING_EMAIL_ADDRESS_W_DASL \
   "http://schemas.microsoft.com/mapi/proptag/0x0065001F"
 
+#define PR_SENDER_NAME_W_DASL \
+  "http://schemas.microsoft.com/mapi/proptag/0x0C1A001F"
+#define PR_SENT_REPRESENTING_NAME_W_DASL \
+  "http://schemas.microsoft.com/mapi/proptag/0x0042001F"
+
 #define DISTRIBUTION_LIST_ADDRESS_ENTRY_TYPE 11
 
+typedef std::shared_ptr<IDispatch> shared_disp_t;
+
+/* Function to contain the gpgol_release macro */
+void release_disp (LPDISPATCH obj);
+
+#define MAKE_SHARED(X) shared_disp_t (X, &release_disp)
+
 /* Return the malloced name of an COM+ object.  */
 char *get_object_name (LPUNKNOWN obj);
 
@@ -194,6 +211,12 @@ HWND get_oom_context_window (LPDISPATCH context);
 std::vector<std::string> get_oom_recipients (LPDISPATCH recipients,
                                              bool *r_err = nullptr);
 
+/* Same as above but include the AddrEntry object in the result.
+   Caller needs to release the AddrEntry. */
+std::vector<std::pair<std::string, shared_disp_t> >
+get_oom_recipients_with_addrEntry (LPDISPATCH recipients,
+                                   bool *r_err = nullptr);
+
 /* Add an attachment to a dispatcher */
 int
 add_oom_attachment (LPDISPATCH disp, const wchar_t* inFile,
@@ -310,14 +333,13 @@ invoke_oom_method_with_parms (LPDISPATCH pDisp, const char *name,
 LPMAPISESSION
 get_oom_mapi_session (void);
 
-/* Ensure a category of the name name exists in
-  the session for the Mail mail.
+/* Ensure a category of the name name exists.
 
   Creates the category with the specified color if required.
 
   returns 0 on success. */
 void
-ensure_category_exists (LPDISPATCH mail, const char *category, int color);
+ensure_category_exists (const char *category, int color);
 
 /* Add a category to a mail if it is not already added. */
 int
@@ -325,7 +347,21 @@ add_category (LPDISPATCH mail, const char *category);
 
 /* Remove a category from a mail if it was added. */
 int
-remove_category (LPDISPATCH mail, const char *category);
+remove_category (LPDISPATCH mail, const char *category, bool exactMatch);
+
+/* Create the category */
+int
+create_category (LPDISPATCH categories, const char *category, int color);
+
+/* Delete a category from the store. */
+int delete_category (LPDISPATCH store, const char *category);
+
+/* Delete categories starting with "string" from all stores. */
+void delete_all_categories_starting_with (const char *string);
+
+/* Iterate over application stores and return the one with the ID
+   @storeID */
+LPDISPATCH get_store_for_id (const char *storeID);
 
 /* Get a unique identifier for a mail object. The
    uuid is a custom property. If create is set
@@ -385,4 +421,10 @@ HRESULT gpgol_openProperty (LPMAPIPROP obj, ULONG ulPropTag, LPCIID lpiid,
 
 /* Check if the preview pane in the explorer is visible */
 bool is_preview_pane_visible (LPDISPATCH explorer);
+
+/* Find or add a text user property with that name. */
+LPDISPATCH find_or_add_text_prop (LPDISPATCH props, const char *name);
+
+/* Find a user property and return it if found. */
+LPDISPATCH find_user_prop (LPDISPATCH props, const char *name);
 #endif /*OOMHELP_H*/
