@@ -64,13 +64,15 @@ using namespace GpgME;
 static bool
 expect_no_headers (msgtype_t type)
 {
-  return type != MSGTYPE_GPGOL_MULTIPART_SIGNED;
+  TSTART;
+  TRETURN type != MSGTYPE_GPGOL_MULTIPART_SIGNED;
 }
 
 static bool
 expect_no_mime (msgtype_t type)
 {
-  return type == MSGTYPE_GPGOL_PGP_MESSAGE ||
+  TSTART;
+  TRETURN type == MSGTYPE_GPGOL_PGP_MESSAGE ||
          type == MSGTYPE_GPGOL_CLEAR_SIGNED;
 }
 
@@ -82,11 +84,13 @@ ParseController::ParseController(LPSTREAM instream, msgtype_t type):
     m_type (type),
     m_block_html (false)
 {
+  TSTART;
   memdbg_ctor ("ParseController");
-  log_mime_parser ("%s:%s: Creating parser for stream: %p of type %i"
+  log_data ("%s:%s: Creating parser for stream: %p of type %i"
                    " expect no headers: %i expect no mime: %i",
                    SRCNAME, __func__, instream, type,
                    expect_no_headers (type), expect_no_mime (type));
+  TRETURN;
 }
 #endif
 
@@ -97,17 +101,21 @@ ParseController::ParseController(FILE *instream, msgtype_t type):
     m_type (type),
     m_block_html (false)
 {
+  TSTART;
   memdbg_ctor ("ParseController");
-  log_mime_parser ("%s:%s: Creating parser for stream: %p of type %i",
+  log_data ("%s:%s: Creating parser for stream: %p of type %i",
                    SRCNAME, __func__, instream, type);
+  TRETURN;
 }
 
 ParseController::~ParseController()
 {
+  TSTART;
   log_debug ("%s:%s", SRCNAME, __func__);
   memdbg_dtor ("ParseController");
   delete m_inputprovider;
   delete m_outputprovider;
+  TRETURN;
 }
 
 static void
@@ -141,15 +149,17 @@ operation_for_type(msgtype_t type, bool *decrypt,
 static bool
 is_smime (Data &data)
 {
+  TSTART;
   data.seek (0, SEEK_SET);
   auto id = data.type();
   data.seek (0, SEEK_SET);
-  return id == Data::CMSSigned || id == Data::CMSEncrypted;
+  TRETURN id == Data::CMSSigned || id == Data::CMSEncrypted;
 }
 
 static std::string
 format_recipients(GpgME::DecryptionResult result, Protocol protocol)
 {
+  TSTART;
   std::string msg;
   for (const auto recipient: result.recipients())
     {
@@ -168,12 +178,13 @@ format_recipients(GpgME::DecryptionResult result, Protocol protocol)
       }
       msg += std::string("<br/>") + _("Unknown Key:") + " 0x" + recipient.keyID();
     }
-  return msg;
+  TRETURN msg;
 }
 
 static std::string
 format_error(GpgME::DecryptionResult result, Protocol protocol)
 {
+  TSTART;
   char *buf;
   bool no_sec = false;
   std::string msg;
@@ -226,22 +237,26 @@ format_error(GpgME::DecryptionResult result, Protocol protocol)
     {
       log_error ("%s:%s:Failed to Format error.",
                  SRCNAME, __func__);
-      return "Failed to Format error.";
+      TRETURN "Failed to Format error.";
     }
-  memdbg_alloc (buf);
   msg = buf;
-  return msg;
+  memdbg_alloc (buf);
+  xfree (buf);
+  TRETURN msg;
 }
 
 void
 ParseController::setSender(const std::string &sender)
 {
+  TSTART;
   m_sender = sender;
+  TRETURN;
 }
 
 static bool
 is_valid_chksum(const GpgME::Signature &sig)
 {
+  TSTART;
   const auto sum = sig.summary();
   static unsigned int valid_mask = (unsigned int) (
       GpgME::Signature::Valid |
@@ -253,14 +268,14 @@ is_valid_chksum(const GpgME::Signature &sig)
       GpgME::Signature::CrlTooOld |
       GpgME::Signature::TofuConflict );
 
-  return sum & valid_mask;
+  TRETURN sum & valid_mask;
 }
 
 
 /* Note on stability:
 
    Experiments have shown that we can have a crash if parse
-   returns at time that is not good for the state of Outlook.
+   Returns at time that is not good for the state of Outlook.
 
    This happend in my test instance after a delay of > 1s < 3s
    with a < 1% chance :-/
@@ -272,6 +287,7 @@ is_valid_chksum(const GpgME::Signature &sig)
 void
 ParseController::parse()
 {
+  TSTART;
   // Wrap the input stream in an attachment / GpgME Data
   Protocol protocol;
   bool decrypt, verify;
@@ -321,11 +337,11 @@ ParseController::parse()
       memdbg_alloc (buf);
       m_error = buf;
       xfree (buf);
-      return;
+      TRETURN;
     }
 
   /* Maybe a different option for this ? */
-  if (opt.autoresolve)
+  if (opt.autoretrieve)
     {
       ctx->setFlag("auto-key-retrieve", "1");
     }
@@ -343,10 +359,11 @@ ParseController::parse()
              decrypt, verify,
              protocol == OpenPGP ? "OpenPGP" :
              protocol == CMS ? "CMS" : "Unknown",
-             m_sender.empty() ? "none" : m_sender.c_str(), inputType);
+             m_sender.empty() ? "none" : anonstr (m_sender.c_str()), inputType);
   if (decrypt)
     {
       input.seek (0, SEEK_SET);
+      TRACEPOINT;
       auto combined_result = ctx->decryptAndVerify(input, output);
       log_debug ("%s:%s:%p decrypt / verify done.",
                  SRCNAME, __func__, this);
@@ -359,6 +376,7 @@ ParseController::parse()
           is_smime (output) ||
           output.type() == Data::Type::PGPSigned)
         {
+          TRACEPOINT;
           /* There is a signature in the output. So we have
              to verify it now as an extra step. */
           input = Data (m_outputprovider);
@@ -367,11 +385,13 @@ ParseController::parse()
           m_outputprovider = new MimeDataProvider();
           output = Data(m_outputprovider);
           verify = true;
+          TRACEPOINT;
         }
       else
         {
           verify = false;
         }
+      TRACEPOINT;
       if (m_decrypt_result.error () || m_decrypt_result.isNull () ||
           m_decrypt_result.error ().isCanceled ())
         {
@@ -380,11 +400,13 @@ ParseController::parse()
     }
   if (verify)
     {
-      const auto sig = m_inputprovider->signature();
+      TRACEPOINT;
+      GpgME::Data *sig = m_inputprovider->signature();
       input.seek (0, SEEK_SET);
       if (sig)
         {
           sig->seek (0, SEEK_SET);
+          TRACEPOINT;
           m_verify_result = ctx->verifyDetachedSignature(*sig, input);
           log_debug ("%s:%s:%p verify done.",
                      SRCNAME, __func__, this);
@@ -400,7 +422,9 @@ ParseController::parse()
         }
       else
         {
+          TRACEPOINT;
           m_verify_result = ctx->verifyOpaqueSignature(input, output);
+          TRACEPOINT;
 
           const auto sigs = m_verify_result.signatures();
           bool allBad = sigs.size();
@@ -472,6 +496,7 @@ ParseController::parse()
   bool ultimate_keys_queried = false;
   for (const auto sig: m_verify_result.signatures())
     {
+      TRACEPOINT;
       has_valid_encrypted_checksum = is_valid_chksum (sig);
 
       KeyCache::instance ()->update (sig.fingerprint (), protocol);
@@ -487,6 +512,7 @@ ParseController::parse()
           get_ultimate_keys ();
           ultimate_keys_queried = true;
         }
+      TRACEPOINT;
     }
 
   if (protocol == Protocol::CMS && decrypt && !m_decrypt_result.error() &&
@@ -497,9 +523,10 @@ ParseController::parse()
       m_block_html = true;
     }
 
-  if (opt.enable_debug)
+  if (opt.enable_debug & DBG_DATA)
     {
       std::stringstream ss;
+      TRACEPOINT;
       ss << m_decrypt_result << '\n' << m_verify_result;
       for (const auto sig: m_verify_result.signatures())
         {
@@ -516,73 +543,89 @@ ParseController::parse()
         }
        log_debug ("Decrypt / Verify result: %s", ss.str().c_str());
     }
+  else
+    {
+      log_debug ("%s:%s:%p Decrypt / verify done errs: %i / %i numsigs: %i.",
+                 SRCNAME, __func__, this, m_decrypt_result.error().code(),
+                 m_verify_result.error().code(), m_verify_result.numSignatures());
+    }
   TRACEPOINT;
 
-  return;
+  if (m_outputprovider)
+    {
+      m_outputprovider->finalize ();
+    }
+
+  TRETURN;
 }
 
 const std::string
 ParseController::get_html_body () const
 {
+  TSTART;
   if (m_outputprovider)
     {
-      return m_outputprovider->get_html_body ();
+      TRETURN m_outputprovider->get_html_body ();
     }
   else
     {
-      return std::string();
+      TRETURN std::string();
     }
 }
 
 const std::string
 ParseController::get_body () const
 {
+  TSTART;
   if (m_outputprovider)
     {
-      return m_outputprovider->get_body ();
+      TRETURN m_outputprovider->get_body ();
     }
   else
     {
-      return std::string();
+      TRETURN std::string();
     }
 }
 
 const std::string
 ParseController::get_body_charset() const
 {
+  TSTART;
   if (m_outputprovider)
     {
-      return m_outputprovider->get_body_charset();
+      TRETURN m_outputprovider->get_body_charset();
     }
   else
     {
-      return std::string();
+      TRETURN std::string();
     }
 }
 
 const std::string
 ParseController::get_html_charset() const
 {
+  TSTART;
   if (m_outputprovider)
     {
-      return m_outputprovider->get_html_charset();
+      TRETURN m_outputprovider->get_html_charset();
     }
   else
     {
-      return std::string();
+      TRETURN std::string();
     }
 }
 
 std::vector<std::shared_ptr<Attachment> >
 ParseController::get_attachments() const
 {
+  TSTART;
   if (m_outputprovider)
     {
-      return m_outputprovider->get_attachments();
+      TRETURN m_outputprovider->get_attachments();
     }
   else
     {
-      return std::vector<std::shared_ptr<Attachment> >();
+      TRETURN std::vector<std::shared_ptr<Attachment> >();
     }
 }
 
@@ -591,13 +634,14 @@ GPGRT_LOCK_DEFINE(keylist_lock);
 std::vector<Key>
 ParseController::get_ultimate_keys()
 {
+  TSTART;
   static bool s_keys_listed;
   static std::vector<Key> s_ultimate_keys;
-  gpgrt_lock_lock (&keylist_lock);
+  gpgol_lock (&keylist_lock);
   if (s_keys_listed)
     {
-      gpgrt_lock_unlock (&keylist_lock);
-      return s_ultimate_keys;
+      gpgol_unlock (&keylist_lock);
+      TRETURN s_ultimate_keys;
     }
   log_debug ("%s:%s: Starting keylisting.",
              SRCNAME, __func__);
@@ -607,8 +651,8 @@ ParseController::get_ultimate_keys()
       /* Maybe PGP broken and not S/MIME */
       log_error ("%s:%s: broken installation no ctx.",
                  SRCNAME, __func__);
-      gpgrt_lock_unlock (&keylist_lock);
-      return s_ultimate_keys;
+      gpgol_unlock (&keylist_lock);
+      TRETURN s_ultimate_keys;
     }
   ctx->setKeyListMode (KeyListMode::Local);
   Error err;
@@ -617,8 +661,8 @@ ParseController::get_ultimate_keys()
     {
       log_error ("%s:%s: Failed to start keylisting err: %i: %s",
                  SRCNAME, __func__, err.code (), err.asString());
-      gpgrt_lock_unlock (&keylist_lock);
-      return s_ultimate_keys;
+      gpgol_unlock (&keylist_lock);
+      TRETURN s_ultimate_keys;
     }
   TRACEPOINT;
   while (!err)
@@ -643,8 +687,8 @@ ParseController::get_ultimate_keys()
               s_ultimate_keys.push_back (key);
               log_debug ("%s:%s: Adding ultimate uid.",
                          SRCNAME, __func__);
-              log_mime_parser ("%s:%s: Added uid %s.",
-                               SRCNAME, __func__, uid.id());
+              log_data ("%s:%s: Added uid %s.",
+                        SRCNAME, __func__, uid.id());
               break;
             }
         }
@@ -654,6 +698,6 @@ ParseController::get_ultimate_keys()
              SRCNAME, __func__);
 
   s_keys_listed = true;
-  gpgrt_lock_unlock (&keylist_lock);
-  return s_ultimate_keys;
+  gpgol_unlock (&keylist_lock);
+  TRETURN s_ultimate_keys;
 }
