@@ -43,6 +43,7 @@
 #include "filetype.h"
 #include "mail.h"
 #include "dispcache.h"
+#include "addressbook.h"
 
 #include <gpgme++/context.h>
 #include <gpgme++/data.h>
@@ -531,6 +532,13 @@ get_mail_from_control (LPDISPATCH ctrl, bool *none_selected)
   if (!uid)
     {
       LPMESSAGE msg = get_oom_base_message (mailitem);
+      if (!msg)
+        {
+          log_debug ("%s:%s: Failed to get message for %p",
+                   SRCNAME, __func__, mailitem);
+          gpgol_release (mailitem);
+          return NULL;
+        }
       uid = mapi_get_uid (msg);
       gpgol_release (msg);
       if (!uid)
@@ -688,9 +696,13 @@ HRESULT launch_cert_details (LPDISPATCH ctrl)
       gpgrt_asprintf (&buf, _("The message was not cryptographically signed.\n"
                       "There is no additional information available if it "
                       "was actually sent by '%s' or if someone faked the sender address."), mail->getSender_o ().c_str());
-      MessageBox (NULL, buf, _("GpgOL"),
-                  MB_ICONINFORMATION|MB_OK);
+      wchar_t *w_msg = utf8_to_wchar (buf);
+      wchar_t *w_title = utf8_to_wchar (_("GpgOL"));
+      MessageBoxW (NULL, w_msg, w_title,
+                   MB_ICONINFORMATION|MB_OK);
       xfree (buf);
+      xfree (w_msg);
+      xfree (w_title);
       return S_OK;
     }
 
@@ -748,11 +760,13 @@ HRESULT launch_cert_details (LPDISPATCH ctrl)
 
   if (showError)
     {
-      MessageBox (NULL,
-                  _("Could not find Kleopatra.\n"
-                  "Please reinstall Gpg4win with the Kleopatra component enabled."),
-                  _("GpgOL"),
-                  MB_ICONINFORMATION|MB_OK);
+      wchar_t *w_title = utf8_to_wchar (_("GpgOL"));
+      wchar_t *w_msg = utf8_to_wchar (_("Could not find Kleopatra.\n"
+                  "Please reinstall Gpg4win with the Kleopatra component enabled."));
+      MessageBoxW (NULL, w_msg, w_title,
+                   MB_ICONINFORMATION|MB_OK);
+      xfree (w_title);
+      xfree (w_msg);
     }
   return S_OK;
 }
@@ -795,3 +809,36 @@ HRESULT print_decrypted (LPDISPATCH ctrl)
   invoke_oom_method (mail->item(), "PrintOut", NULL);
   return S_OK;
 }
+
+HRESULT open_contact_key (LPDISPATCH ctrl)
+{
+  if (!ctrl)
+    {
+      log_error ("%s:%s:%i", SRCNAME, __func__, __LINE__);
+      return E_FAIL;
+    }
+  LPDISPATCH inspector = NULL;
+  HRESULT hr = getContext (ctrl, &inspector);
+
+  if (hr)
+    {
+      log_error ("%s:%s:%i : hresult %lx", SRCNAME, __func__, __LINE__,
+                 hr);
+      return S_OK;
+    }
+
+  /* Context is assumed to be the Insepector */
+  LPDISPATCH contact = get_oom_object (inspector, "CurrentItem");
+  gpgol_release (inspector);
+
+  if (!contact)
+    {
+      TRACEPOINT;
+      return S_OK;
+    }
+
+  Addressbook::edit_key_o (contact);
+
+  gpgol_release (contact);
+  return S_OK;
+}
