@@ -49,6 +49,7 @@
 #define ID_CMD_SIGN_ENCRYPT_MIME_EX 33
 #define ID_CMD_PRINT_DECRYPTED 34
 #define ID_GET_IS_CRYPTO_MAIL 35
+#define ID_CMD_OPEN_CONTACT_KEY 36
 
 #define ID_BTN_DECRYPT           IDI_DECRYPT_16_PNG
 #define ID_BTN_DECRYPT_LARGE     IDI_DECRYPT_48_PNG
@@ -85,4 +86,6 @@ HRESULT ribbon_loaded (LPDISPATCH ctrl);
 HRESULT get_is_crypto_mail (LPDISPATCH ctrl, VARIANT *result);
 /* Print out the decrypted mail */
 HRESULT print_decrypted (LPDISPATCH ctrl);
+/* Open key configuration for a contact */
+HRESULT open_contact_key (LPDISPATCH ctrl);
 #endif
