@@ -26,6 +26,7 @@
 #include "mail.h"
 #include "gpgoladdin.h"
 #include "wks-helper.h"
+#include "addressbook.h"
 
 #include <stdio.h>
 
@@ -41,6 +42,7 @@ gpgol_window_proc (HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 //  log_debug ("WMG: %x", (unsigned int) message);
   if (message == WM_USER + 42)
     {
+      TSTART;
       wm_ctx_t *ctx = (wm_ctx_t *) lParam;
       log_debug ("%s:%s: Recieved user msg: %i",
                  SRCNAME, __func__, ctx->wmsg_type);
@@ -53,10 +55,10 @@ gpgol_window_proc (HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
                 {
                   log_debug ("%s:%s: Parsing done for mail which is gone.",
                              SRCNAME, __func__);
-                  break;
+                  TBREAK;
                 }
               mail->parsing_done();
-              break;
+              TBREAK;
             }
           case (RECIPIENT_ADDED):
             {
@@ -65,10 +67,10 @@ gpgol_window_proc (HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
                 {
                   log_debug ("%s:%s: Recipient add for mail which is gone.",
                              SRCNAME, __func__);
-                  break;
+                  TBREAK;
                 }
               mail->locateKeys_o ();
-              break;
+              TBREAK;
             }
           case (REVERT_MAIL):
             {
@@ -77,7 +79,7 @@ gpgol_window_proc (HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
                 {
                   log_debug ("%s:%s: Revert mail for mail which is gone.",
                              SRCNAME, __func__);
-                  break;
+                  TBREAK;
                 }
 
               mail->setNeedsSave (true);
@@ -96,7 +98,7 @@ gpgol_window_proc (HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
               mail->updateBody_o ();
               log_debug ("%s:%s: Revert mail done.",
                          SRCNAME, __func__);
-              break;
+              TBREAK;
             }
           case (INVALIDATE_UI):
             {
@@ -114,14 +116,14 @@ gpgol_window_proc (HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
                              " Ignoring it",
                              SRCNAME, __func__);
                 }
-              break;
+              TBREAK;
             }
           case (INVALIDATE_LAST_MAIL):
             {
               log_debug ("%s:%s: clearing last mail",
                          SRCNAME, __func__);
               Mail::clearLastMail ();
-              break;
+              TBREAK;
             }
           case (CLOSE):
             {
@@ -130,14 +132,16 @@ gpgol_window_proc (HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
                 {
                   log_debug ("%s:%s: Close for mail which is gone.",
                              SRCNAME, __func__);
-                  break;
+                  TBREAK;
                 }
               mail->refCurrentItem();
+              Mail::closeInspector_o (mail);
+              TRACEPOINT;
               Mail::close (mail);
               log_debug ("%s:%s: Close finished.",
                          SRCNAME, __func__);
               mail->releaseCurrentItem();
-              break;
+              TBREAK;
             }
           case (CRYPTO_DONE):
             {
@@ -146,7 +150,7 @@ gpgol_window_proc (HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
                 {
                   log_debug ("%s:%s: Crypto done for mail which is gone.",
                              SRCNAME, __func__);
-                  break;
+                  TBREAK;
                 }
               // modify the mail.
               if (mail->cryptState () == Mail::NeedsUpdateInOOM)
@@ -163,11 +167,48 @@ gpgol_window_proc (HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
                              SRCNAME, __func__, mail);
                 }
               // Finaly this should pass.
-              invoke_oom_method (mail->item (), "Send", NULL);
+              if (invoke_oom_method (mail->item (), "Send", NULL))
+                {
+                  log_error ("%s:%s: Send failed for %p. "
+                             "Trying SubmitMessage instead.\n"
+                             "This will likely crash.",
+                             SRCNAME, __func__, mail);
+
+                  /* What we do here is similar to the T3656 workaround
+                     in mailitem-events.cpp. In our tests this works but
+                     is unstable. So we only use it as a very very last
+                     resort. */
+                  auto mail_message = get_oom_base_message (mail->item());
+                  if (!mail_message)
+                    {
+                      gpgol_bug (mail->getWindow (),
+                                 ERR_GET_BASE_MSG_FAILED);
+                      TBREAK;
+                    }
+                  // It's important we use the _base_ message here.
+                  mapi_save_changes (mail_message, FORCE_SAVE);
+                  HRESULT hr = mail_message->SubmitMessage(0);
+                  gpgol_release (mail_message);
+
+                  if (hr == S_OK)
+                    {
+                      do_in_ui_thread_async (CLOSE, (LPVOID) mail);
+                    }
+                  else
+                    {
+                      log_error ("%s:%s: SubmitMessage Failed hr=0x%lx.",
+                                 SRCNAME, __func__, hr);
+                      gpgol_bug (mail->getWindow (),
+                                 ERR_SEND_FALLBACK_FAILED);
+                    }
+                }
+              else
+                {
+                  mail->releaseCurrentItem ();
+                }
               log_debug ("%s:%s:  Send for %p completed.",
                          SRCNAME, __func__, mail);
-              mail->releaseCurrentItem();
-              break;
+              TBREAK;
             }
           case (BRING_TO_FRONT):
             {
@@ -183,13 +224,13 @@ gpgol_window_proc (HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
                   log_debug ("%s:%s: No active window found for bring to front.",
                              SRCNAME, __func__);
                 }
-              break;
+              TBREAK;
             }
           case (WKS_NOTIFY):
             {
               WKSHelper::instance ()->notify ((const char *) ctx->data);
               xfree (ctx->data);
-              break;
+              TBREAK;
             }
           case (CLEAR_REPLY_FORWARD):
             {
@@ -198,11 +239,11 @@ gpgol_window_proc (HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
                 {
                   log_debug ("%s:%s: Clear reply forward for mail which is gone.",
                              SRCNAME, __func__);
-                  break;
+                  TBREAK;
                 }
               mail->wipe_o (true);
               mail->removeAllAttachments_o ();
-              break;
+              TBREAK;
             }
           case (DO_AUTO_SECURE):
             {
@@ -211,10 +252,10 @@ gpgol_window_proc (HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
                 {
                   log_debug ("%s:%s: DO_AUTO_SECURE for mail which is gone.",
                              SRCNAME, __func__);
-                  break;
+                  TBREAK;
                 }
               mail->setDoAutosecure_m (true);
-              break;
+              TBREAK;
             }
           case (DONT_AUTO_SECURE):
             {
@@ -223,16 +264,24 @@ gpgol_window_proc (HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
                 {
                   log_debug ("%s:%s: DO_AUTO_SECURE for mail which is gone.",
                              SRCNAME, __func__);
-                  break;
+                  TBREAK;
                 }
               mail->setDoAutosecure_m (false);
-              break;
+              TBREAK;
+            }
+          case (CONFIG_KEY_DONE):
+            {
+              log_debug ("%s:%s: Key configuration done.",
+                         SRCNAME, __func__);
+
+              Addressbook::update_key_o (ctx->data);
+              TBREAK;
             }
           default:
             log_debug ("%s:%s: Unknown msg %x",
                        SRCNAME, __func__, ctx->wmsg_type);
         }
-        return 0;
+        TRETURN 0;
     }
   return DefWindowProc(hWnd, message, wParam, lParam);
 }
@@ -240,11 +289,12 @@ gpgol_window_proc (HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 HWND
 create_responder_window ()
 {
+  TSTART;
   size_t cls_name_len = strlen(RESPONDER_CLASS_NAME) + 1;
   char cls_name[cls_name_len];
   if (g_responder_window)
     {
-      return g_responder_window;
+      TRETURN g_responder_window;
     }
   /* Create Window wants a mutable string as the first parameter */
   snprintf (cls_name, cls_name_len, "%s", RESPONDER_CLASS_NAME);
@@ -264,12 +314,13 @@ create_responder_window ()
   g_responder_window = CreateWindow (cls_name, RESPONDER_CLASS_NAME, 0, 0, 0,
                                      0, 0, 0, (HMENU) 0,
                                      (HINSTANCE) GetModuleHandle(NULL), 0);
-  return g_responder_window;
+  TRETURN g_responder_window;
 }
 
 static int
 send_msg_to_ui_thread (wm_ctx_t *ctx)
 {
+  TSTART;
   size_t cls_name_len = strlen(RESPONDER_CLASS_NAME) + 1;
   char cls_name[cls_name_len];
   snprintf (cls_name, cls_name_len, "%s", RESPONDER_CLASS_NAME);
@@ -279,15 +330,16 @@ send_msg_to_ui_thread (wm_ctx_t *ctx)
   {
     log_error ("%s:%s: Failed to find responder window.",
                SRCNAME, __func__);
-    return -1;
+    TRETURN -1;
   }
   SendMessage (responder, WM_USER + 42, 0, (LPARAM) ctx);
-  return 0;
+  TRETURN 0;
 }
 
 int
 do_in_ui_thread (gpgol_wmsg_type type, void *data)
 {
+  TSTART;
   wm_ctx_t ctx = {NULL, UNKNOWN, 0, 0};
   ctx.wmsg_type = type;
   ctx.data = data;
@@ -297,14 +349,15 @@ do_in_ui_thread (gpgol_wmsg_type type, void *data)
 
   if (send_msg_to_ui_thread (&ctx))
     {
-      return -1;
+      TRETURN -1;
     }
-  return ctx.err;
+  TRETURN ctx.err;
 }
 
 static DWORD WINAPI
 do_async (LPVOID arg)
 {
+  TSTART;
   wm_ctx_t *ctx = (wm_ctx_t*) arg;
   log_debug ("%s:%s: Do async with type %i after %i ms",
              SRCNAME, __func__, ctx ? ctx->wmsg_type : -1,
@@ -315,18 +368,20 @@ do_async (LPVOID arg)
     }
   send_msg_to_ui_thread (ctx);
   xfree (ctx);
-  return 0;
+  TRETURN 0;
 }
 
 void
 do_in_ui_thread_async (gpgol_wmsg_type type, void *data, int delay)
 {
+  TSTART;
   wm_ctx_t *ctx = (wm_ctx_t *) xcalloc (1, sizeof (wm_ctx_t));
   ctx->wmsg_type = type;
   ctx->data = data;
   ctx->delay = delay;
 
   CloseHandle (CreateThread (NULL, 0, do_async, (LPVOID) ctx, 0, NULL));
+  TRETURN;
 }
 
 LRESULT CALLBACK
@@ -352,7 +407,7 @@ gpgol_hook(int code, WPARAM wParam, LPARAM lParam)
         if (!GpgolAddin::get_instance() || !GpgolAddin::get_instance ()->get_application())
           {
             TRACEPOINT;
-            break;
+            TBREAK;
           }
         LPDISPATCH explorers = get_oom_object (GpgolAddin::get_instance ()->get_application(),
                                                "Explorers");
@@ -361,7 +416,7 @@ gpgol_hook(int code, WPARAM wParam, LPARAM lParam)
           {
             log_error ("%s:%s: No explorers object",
                        SRCNAME, __func__);
-            break;
+            TBREAK;
           }
         int count = get_oom_int (explorers, "Count");
 
@@ -370,7 +425,7 @@ gpgol_hook(int code, WPARAM wParam, LPARAM lParam)
             log_debug ("%s:%s: More then one explorer. Not shutting down.",
                        SRCNAME, __func__);
             gpgol_release (explorers);
-            break;
+            TBREAK;
           }
 
         LPDISPATCH explorer = get_oom_object (explorers, "Item(1)");
@@ -379,11 +434,11 @@ gpgol_hook(int code, WPARAM wParam, LPARAM lParam)
         if (!explorer)
           {
             TRACEPOINT;
-            break;
+            TBREAK;
           }
 
         /* Casting to LPOLEWINDOW and calling GetWindow
-           succeeded in Outlook 2016 but always returned
+           succeeded in Outlook 2016 but always TRETURNed
            the number 1. So we need this hack. */
         char *caption = get_oom_string (explorer, "Caption");
         gpgol_release (explorer);
@@ -391,7 +446,7 @@ gpgol_hook(int code, WPARAM wParam, LPARAM lParam)
           {
             log_debug ("%s:%s: No caption.",
                        SRCNAME, __func__);
-            break;
+            TBREAK;
           }
         /* rctrl_renwnd32 is the window class of outlook. */
         HWND hwnd = FindWindowExA(NULL, lastChild, "rctrl_renwnd32",
@@ -404,9 +459,9 @@ gpgol_hook(int code, WPARAM wParam, LPARAM lParam)
                        "Shutting down.",
                        SRCNAME, __func__);
             GpgolAddin::get_instance ()->shutdown();
-            break;
+            TBREAK;
           }
-        break;
+        TBREAK;
       }
      case WM_SYSCOMMAND:
         /*
@@ -432,7 +487,8 @@ gpgol_hook(int code, WPARAM wParam, LPARAM lParam)
 HHOOK
 create_message_hook()
 {
-  return SetWindowsHookEx (WH_CALLWNDPROC,
+  TSTART;
+  TRETURN SetWindowsHookEx (WH_CALLWNDPROC,
                            gpgol_hook,
                            NULL,
                            GetCurrentThreadId());
@@ -445,15 +501,16 @@ static bool invalidation_in_progress;
 DWORD WINAPI
 delayed_invalidate_ui (LPVOID minsleep)
 {
+  TSTART;
   if (invalidation_in_progress)
     {
       log_debug ("%s:%s: Invalidation canceled as it is in progress.",
                  SRCNAME, __func__);
-      return 0;
+      TRETURN 0;
     }
   TRACEPOINT;
   invalidation_in_progress = true;
-  gpgrt_lock_lock(&invalidate_lock);
+  gpgol_lock(&invalidate_lock);
 
   int sleep_ms = (intptr_t)minsleep;
   Sleep (sleep_ms);
@@ -473,30 +530,34 @@ delayed_invalidate_ui (LPVOID minsleep)
   do_in_ui_thread (INVALIDATE_UI, nullptr);
   TRACEPOINT;
   invalidation_in_progress = false;
-  gpgrt_lock_unlock(&invalidate_lock);
-  return 0;
+  gpgol_unlock(&invalidate_lock);
+  TRETURN 0;
 }
 
 DWORD WINAPI
 close_mail (LPVOID mail)
 {
+  TSTART;
   do_in_ui_thread (CLOSE, mail);
-  return 0;
+  TRETURN 0;
 }
 
 void
 blockInv()
 {
+  TSTART;
   invalidation_blocked++;
-  log_oom_extra ("%s:%s: Invalidation block count %i",
+  log_oom ("%s:%s: Invalidation block count %i",
                  SRCNAME, __func__, invalidation_blocked);
+  TRETURN;
 }
 
 void
 unblockInv()
 {
+  TSTART;
   invalidation_blocked--;
-  log_oom_extra ("%s:%s: Invalidation block count %i",
+  log_oom ("%s:%s: Invalidation block count %i",
                  SRCNAME, __func__, invalidation_blocked);
 
   if (invalidation_blocked < 0)
@@ -505,4 +566,5 @@ unblockInv()
                  SRCNAME, __func__);
       invalidation_blocked = 0;
     }
+  TRETURN;
 }
